angular.module('gurillaApp.controllers')

/* @screen: List of channels */
.controller('ChannelsCtrl', function(
$rootScope, $scope, $state, $stateParams, $ionicHistory,$ionicListDelegate,
$ionicModal,$timeout, $ionicActionSheet, $ionicLoading, $ionicPopup,ChannelCrud,nzTour,Tutorials,Helper,
LocalData, Channels)
{

  $rootScope.hideTabs = false;
  
  $scope.search = { keyword: ''};
  
  //set var for timer
  var timer;
  //call this function whenever user stop typing
  //http method will call after 0.5 sec

  Helper.showTutorial(Tutorials.channel);

  $scope.$watch("search.keyword", function(newValue, oldValue) {
    if (newValue && newValue.trim() !== '') {
      newValue = newValue.trim();
      oldValue = oldValue.trim();

      if (newValue != oldValue) {
        $timeout.cancel(timer);
        
        //start timer
        $scope.showLoading = true;
        timer = $timeout(function(){
          //start searching
          ChannelCrud.search($scope.search.keyword,function(data){
            $scope.showLoading = false;
            if (data.type == "OK") {
              $scope.suggestions = data.channels;
            }else{
              $scope.message = data.message;  
            }
          });
         }, 500);

      }
    }
  });

  //Reload all the channel from server
  $scope.tryRefresh = function(callback){
    ChannelCrud.read(function(data){
      $scope.$broadcast('scroll.refreshComplete');
      
      if (data.type == "OK") {
        $scope.message = '';
        //create new buffer data
        var c = { created: [], joined: [] };
        //fill with coresponding data return by server
        c.created = data.my_channels;
        c.joined = data.channels;
        //update local data as well as global channels
        $rootScope.channels = Channels.set(c);
      }else{
        $scope.message = data.message;
      }

    });
  };

  $scope.showSearch = function(){
    if(Helper.isTutorialFinished(Tutorials.channel)){
      $scope.channelModal.show()  
    }
    
  };

  $scope.isSubscribed = function(channel){
    for (var i = 0; i < $rootScope.channels.joined.length; i++) {
       return $rootScope.channels.joined[i].id == channel.id;
    }
  };

  $scope.createNewChannel = function(){
    if(Helper.isTutorialFinished(Tutorials.channel)){
      if ($rootScope.channels.created.length >= 2) {
            $ionicPopup.alert({
             title: 'Reached Quota',
             template: 'User may not create more than 2 Channels'
           });
            return false;
          }
          $state.go('menu.tab.channel-create');
    }

    
  };

  $scope.viewChannel = function(channel) {
    $scope.channelModal.hide();
    $state.go('menu.tab.channel-detail',{"channel" : channel});
  };

  $scope.updateChannel = function(type,channel) {
    $scope.channelModal.hide();
    $state.go('menu.tab.manage-channel',{"channel":channel,"type":type});
  };
   //create new event
  $scope.createNewEvent = function(channel) {
    $ionicListDelegate.closeOptionButtons();
    $scope.channelModal.hide();
    $state.go("menu.tab.channel-create-event",{"type":"channels" , "object" : channel});
  };
  
  //check rootScope first
  if (!$rootScope.channels) {
    $rootScope.channels = Channels.all();
    
    //check rootScope channels return from local data
    if (!$rootScope.channels) {
      var loading = $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Loading channels..'
      });
      $scope.tryRefresh();
    }
  }

  $ionicModal.fromTemplateUrl('templates/modal-search-channels.html', function($ionicModal) {
      $scope.channelModal = $ionicModal;
  }, {
      scope: $scope,
      focusFirstInput: true,
      animation: 'slide-in-up'
  });

})

/* @screen: Create a new channel */
.controller('ChannelCreateCtrl', function(
$rootScope, $scope, $state, $stateParams, $ionicHistory,$timeout,
$ionicModal, $ionicActionSheet, $ionicLoading,$cordovaCamera,ChannelCrud,nzTour,Tutorials,Helper,
LocalData, Channels)
{
  $rootScope.hideTabs = false;
  $scope.newChannel = {};

  
  Helper.showTutorial(Tutorials.channel_create);


  $ionicModal.fromTemplateUrl('templates/modal-channel-about.html', function($ionicModal) {
      $scope.modalAbout = $ionicModal;
  }, {
      scope: $scope,
      focusFirstInput: false,
      animation: 'slide-in-up'
  });

  $scope.showAboutModal = function() {
    if(Helper.isTutorialFinished(Tutorials.channel_create)){
      $rootScope.hideKeyboardAccessoryBar(true);
      $scope.modalData = { channelDescription: $scope.newChannel.description};
      $scope.modalAbout.show();  
    }
    
  };

  $scope.updateAbout = function() {
    $scope.newChannel.description = $scope.modalData.channelDescription;
    $scope.modalAbout.hide();
    $rootScope.hideKeyboardAccessoryBar(false);
  };

  $scope.isSpace = function(name){
   $scope.newChannel.name = name.replace(/\s/g, '');
  };

  $scope.openCameraActionSheet = function(){
    if(Helper.isTutorialFinished(Tutorials.channel_create)){
      var hideSheet = $ionicActionSheet.show({
      buttons: [
        { text: 'Use Camera' },
        { text: 'Choose Photo' }
      ],
      cancelText: 'Cancel',
      cancel: function() {
      },
      buttonClicked: function(index) {
        if (navigator.camera) {
          
          var source = (index == 1)? navigator.camera.PictureSourceType.PHOTOLIBRARY : navigator.camera.PictureSourceType.CAMERA;
          $cordovaCamera.getPicture(Channels.getCameraOptions(source)).then(function(imageURI) {
             var coverPhoto = document.getElementById('coverPhoto');
              $scope.newChannel.photo = imageURI;
              coverPhoto.src = 'data:image/jpeg;base64,'+imageURI;          
          }, function(err) {
            console.log(err);
          });

        } else {
          $scope.message = 'Unable to access camera. Are you on browser?';
        }
        return true;
      }
    });  
    }
    
  };

  $scope.tryCreate = function(){
    
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Creating channel..'
    });
    
    ChannelCrud.create($scope.newChannel,function(data){
      $ionicLoading.hide();
      
      if (data.type == "OK") {
        $scope.message = '';
        Channels.add('created',data.channel,function(channels){
          $rootScope.channels = channels;
          $state.go('menu.tab.channels');
        });
        
      }else{
        $scope.message = data.message;
      }

    });
  };

})

/* @screen: Update a channel */
.controller('ChannelUpdateCtrl', function(
$rootScope, $scope, $state, $stateParams, $ionicHistory,$ionicPopover,$cordovaCamera,$window,$cacheFactory,$timeout,
$ionicModal, $ionicActionSheet, $ionicLoading,$ionicPopup,
ChannelCrud,SocialCrud,MomentCrud,Contacts,CommentCrud,LocalData, Channels,Moments,Helper,Tutorials)

{



  Helper.showTutorial(Tutorials.channel_update);
  
  $rootScope.hideTabs = false;
  //get the channel and type from params
  $scope.channel = $stateParams.channel;
  //get account infomartion
  $scope.account = LocalData.getAccount();
  //get channel type
  var type = $stateParams.type;
  //retrieve the index 
  var index = -1; 

  $scope.$on('$ionicView.enter', function() {
    $scope.moments = [];
    $scope.moments = Moments.getLocalMomentsByChannelID($scope.channel.id);
  });

  $scope.showAboutModal = function() {
    $rootScope.hideKeyboardAccessoryBar(true);
    $scope.modalData = { channelDescription: $scope.channel.description};
    $scope.modalAbout.show();
  };

  $scope.share = function(){
    $scope.settingPopover.hide();
    if ($window.plugins) { 
      $window.plugins.socialsharing.share("Hi guys! I just created an awesome channel for you to subscribe to for my future events & meetups! Its called #"+$scope.channel.name+" at Gurilla. Pls download the app to join! Thanks", null, null , 'http://goo.gl/yWSlC6');
    }
  };

  $scope.showActions = function($event){
    if(Helper.isTutorialFinished(Tutorials.channel_update)){
      $scope.settingPopover.show($event);    
    }
    
  };

  $scope.updateAbout = function() {
    $scope.channel.description = $scope.modalData.channelDescription;
    $scope.modalAbout.hide();
    $rootScope.hideKeyboardAccessoryBar(false);
  };

  //open camera to upload new photo
  $scope.openCameraActionSheet = function(){
    var hideSheet = $ionicActionSheet.show({
      buttons: [
        { text: 'Use Camera' },
        { text: 'Choose Photo' }
      ],
      cancelText: 'Cancel',
      cancel: function() {
      },
      buttonClicked: function(index) {

        if (navigator.camera) {
         //determine source  
         var source = (index == 1)? navigator.camera.PictureSourceType.PHOTOLIBRARY : navigator.camera.PictureSourceType.CAMERA;
         //add new attr to scope channel call newPhoto to diff 
         //so that the photo will be shown as base64
         $cordovaCamera.getPicture(Channels.getCameraOptions(source)).then(function(imageURI) {
             var coverPhoto = document.getElementById('coverPhoto');
              $scope.channel.photo = '';
              $scope.channel.newPhoto = imageURI;
              coverPhoto.src = 'data:image/jpeg;base64,'+imageURI;          
          }, function(err) {
            console.log(err);
          });

        } else {
          $scope.message = 'Unable to access camera. Are you on browser?';
        }

        return true;
      }
    });
  };

  //update the channel 
  //also upate the local data by giving 
  //type (created|joined) , index and channel
  $scope.tryUpdate = function() {
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Updating channel..'
    });

    var channel = angular.copy($scope.channel);

    channel.photo = $scope.channel.newPhoto;
    
    ChannelCrud.update(channel,function(data){
      
      if (data.type == "OK") {
        $scope.message = '';
        
        Channels.update(type,index,data.channel,function(channels){
          $ionicLoading.hide();
          $state.go('menu.tab.channels');
          $rootScope.channels = channels;
        });
      }else{
        $scope.message = data.message;
      }
    });

  };

   $scope.isMe = function(id){
      //this is channel owner , so he can delete all comment;
      return true;
      // return Contacts.isMe(id);
    };

   $scope.viewProfile = function(user){
      $scope.commentsModal.hide();
      if(Contacts.isMe(user.id)){
        $state.go("menu.tab.account");
        return;
      }
      $state.go("menu.tab.account-friend-profile",{accountID : user.id});
    };


   //create new event
  $scope.createNewEvent = function(channel) {
    $scope.settingPopover.hide();
    if ($state.current.name.indexOf("account") > -1) {
      $state.go("menu.tab.account-channel-create-event",{"type":"channels" , "object" : $scope.channel});
      return;  
    }
    $state.go("menu.tab.channel-create-event",{"type":"channels" , "object" : $scope.channel});
  };

  //delete channel and also from localdata
  $scope.removeChannel = function() {

    $scope.settingPopover.hide();

     var confirmPopup = $ionicPopup.confirm({
       title: 'Delete Channel',
       template: 'Are you sure you want to delete this channel?'
     });
     confirmPopup.then(function(res) {
       if (res) {
         
         $ionicLoading.show({
            template: '<i class="icon ion-loading-c"></i> Removing channel..'
         });

         ChannelCrud.delete($scope.channel.id,function(data){
              if (data.type == "OK") {
                $scope.message = '';
                Channels.remove(type,index,function(channels){
                  $ionicLoading.hide();
                  $state.go('menu.tab.channels');
                  $rootScope.channels = channels;
                });

              }else{
                $scope.message = data.message;
              }
          });
       }
     });
  };


  //get all moment related to this channel
  $scope.getMoments = function(useCache){    
    var loading = $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading moments...'
    });
    //calling the exact same method for subscribing channel
    //yet with different params 
    
    //get moments from server
    MomentCrud.getMomentsByChannelID($scope.channel.id, useCache, function(data){
      if (data.type == "OK") {
       $ionicLoading.hide();
       $scope.moments = data.moments;
      }else{
        $scope.message = data.message;
      }
    });
  };

  //view deatil of moment
  $scope.viewPhoto = function(moment){
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading...'
    });
    console.log("View moment");
    console.log(moment);
    SocialCrud.detail(moment.social_id,function(data){
      $ionicLoading.hide(); 
      if(data.type == "OK"){
        $scope.moment = data.social;
        $state.go('menu.tab.account-photo',{"moment" : data.social, "account" : $scope.account , "access" : true, "extra" : {"channel_id" : $scope.channel.id, "from" : "channel"}});
      }else{
        $scope.message = data.message;
      }
    });
   
  };



   $scope.nextCommentPage = 1;
    $scope.viewComments = function(isRefresh){
      if(!Helper.isTutorialFinished(Tutorials.channel_update)){ 
          return;
      }

      if(isRefresh){
        $scope.nextCommentPage = 1;
      }

      if ($scope.channel.socials) {
        $scope.social_id = $scope.channel.socials.id;
      }
      
      if ( $scope.social_id < 1) {
        $scope.message = "Comments can not be shown for this channel";
        return;
      }

      if (!$scope.commentsModal.isShown()) {
          $scope.commentsModal.show();
      }
      

       $ionicLoading.show({
          template: '<i class="icon ion-loading-c"></i> Loading comments...'
       });
       
        CommentCrud.read($scope.social_id,$scope.nextCommentPage,function(data){
          $ionicLoading.hide();  
          if(data.type == "OK"){
            if ($scope.nextCommentPage > 1) {
              $scope.comments = $scope.comments.concat(data.comments.result);
            }else{
              $scope.comments =  data.comments.result;
            }
            
            $scope.nextCommentPage = data.comments.next;
            $scope.$broadcast('scroll.refreshComplete');
          }
        });
    };


    $scope.closeComments = function(){
      $scope.commentsModal.hide();
      $scope.comments = {};
      $scope.nextCommentPage = 1;
    };


   $scope.newComment = {};

   //adding new comment
   $scope.addComment = function(){

     $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Posting...'
     });

     CommentCrud.create($scope.social_id,$scope.newComment,function(data){
        $ionicLoading.hide();  
        if(data.type == "OK"){
          $scope.comments.unshift(data.comment);
          $scope.newComment = {};
          //for increment in event comment count
          $scope.channel.socials.comments.push(data.comment);
        }
      });
    };


    //remove comment
    $scope.removeComment = function(comment){
      var confirmPopup = $ionicPopup.confirm({
         title: 'Delete Comment',
         template: 'You are about to delete the comment ?'
       });
       //delete circle if user click okay
       confirmPopup.then(function(res) {
          if (res) {
              var index = $scope.comments.indexOf(comment);
              $scope.comments.splice(index,1);
              CommentCrud.delete($scope.social_id,comment.id,function(data){
                if (data.type == "OK") {
                  $scope.message = '';
                  $scope.event.socials.comments.splice(0,1);
                }else{
                  $scope.message = data.message;
                }
            });

          }
       });
    };


   //load moments 
  $scope.loadChanel = function(useCache) {
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading channel...'
    });

    ChannelCrud.detail($scope.channel,function(data){
      
      $ionicLoading.hide();

      $scope.$broadcast('scroll.refreshComplete');

      if (data.type == "OK") {

        $scope.message = '';

        $scope.channel = data.channel;

        index = Channels.findChannelIndex( $scope.channel, $rootScope.channels[type]);  
        
        console.log(index);

        var channel = angular.copy(data.channel);
        
        channel.photo = $scope.channel.newPhoto;

        $scope.getMoments(useCache);
      
      }else{
        $scope.message = data.message;
      }
    });

  };

  $scope.loadChanel(false);

  //comments modal template
  $ionicModal.fromTemplateUrl('templates/modal-view-comments.html', function($ionicModal) {
    $scope.commentsModal = $ionicModal;
  }, {
    scope: $scope,
    focusFirstInput: true,
    animation: 'slide-in-up'
  });

  $ionicModal.fromTemplateUrl('templates/modal-channel-about.html', function($ionicModal) {
      $scope.modalAbout = $ionicModal;
  }, {
      scope: $scope,
      focusFirstInput: false,
      animation: 'slide-in-up'
  });

  $ionicPopover.fromTemplateUrl('templates/popover-channel-update-setting.html', {
    scope: $scope
  }).then(function(popover) {
    $scope.settingPopover = popover;
  });

})

/* @screen: View a channel in details */
.controller('ChannelDetailCtrl', function(
$rootScope, $scope, $state, $stateParams, $ionicHistory,$ionicPopup,$ionicPopover,
$ionicModal, $ionicActionSheet, $ionicLoading,$window,ChannelCrud,SocialCrud,MomentCrud,Contacts,CommentCrud,LocalData, Moments, Channels)
{

  $rootScope.hideTabs = false;

  $scope.channel = $stateParams.channel;

  $scope.account = LocalData.getAccount();
  
  // $scope.index = Channels.findChannelIndex($scope.channel, $rootScope.channels.joined);
  // $scope.index = $rootScope.channels.joined.indexOf($scope.channel);

  $scope.$on('$ionicView.enter', function() {
    $scope.moments = [];
    $scope.moments = Moments.getLocalMomentsByChannelID($scope.channel.id);
  });


  $scope.index = -1;

  $scope.social_id = -1;

  if($scope.channel){
    $scope.channel.moreinfo = false;
    $scope.channel.summary = $scope.channel.description.substr(0,200)+"...";
  }


  $scope.share = function(){
    $scope.settingPopover.hide();
    if ($window.plugins) { 
      $window.plugins.socialsharing.share("Hi guys, I found out about this channel #"+$scope.channel.name+" from Gurilla app and thought you should join in the movement as well. Download the app to join in. Thanks!",null, null,'http://goo.gl/yWSlC6');
    }
  };

  $scope.showActions = function($event){
      $scope.settingPopover.show($event);    
  };
  
  $scope.toggleMoreInfo = function(){
    if(!$scope.channel.moreinfo){        
        $scope.channel.summary = $scope.channel.description;
        $scope.channel.moreinfo = true;
    }else{
        $scope.channel.summary = $scope.channel.description.substr(0,200)+"...";
        $scope.channel.moreinfo = false;
    }
  };

  $scope.joinChannel = function(){
    $scope.settingPopover.hide();
    var loading = $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Subscribing channel..'
    });
    
    ChannelCrud.subscribe(false,$scope.channel,function(data){
      if (data.type == "OK") {
       $ionicLoading.hide();

       Channels.add('joined',$scope.channel,function(channels){
          $rootScope.channels = channels;
          $scope.index = $rootScope.channels.joined.length - 1;
          console.log($scope.index);
       });
       
      }else{
        $scope.message = data.message;
      }
    });
  };

  $scope.leaveChannel = function(){
    $scope.settingPopover.hide();
    var loading = $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Unsubscribing channel..'
    });
    //calling the exact same method for subscribing channel
    //yet with different params 
    
    ChannelCrud.subscribe(true,$scope.channel,function(data){
      if (data.type == "OK") {
       $ionicLoading.hide();
       Channels.remove('joined',$scope.index , function(channels){
        $rootScope.channels = channels;
       });
       $state.go('menu.tab.channels');
      }else{
        $scope.message = data.message;
      }
    });
  };

  $scope.viewProfile = function(user){
      $scope.commentsModal.hide();
      if(Contacts.isMe(user.id)){
        $state.go("menu.tab.account");
        return;
      }
      $state.go("menu.tab.account-friend-profile",{accountID : user.id});
    };


   $scope.nextCommentPage = 1;
    $scope.viewComments = function(isRefresh){

      if(isRefresh){
        $scope.nextCommentPage = 1;
      }

      if ($scope.channel.socials) {
        $scope.social_id = $scope.channel.socials.id;
      }


       if ( $scope.social_id < 1) {
        $scope.message = "Comments can not be shown for this channel";
        return;
       }


      if (!$scope.commentsModal.isShown()) {
          $scope.commentsModal.show();
      }
      
     
       $ionicLoading.show({
          template: '<i class="icon ion-loading-c"></i> Loading comments...'
       });
       
        CommentCrud.read($scope.social_id,$scope.nextCommentPage,function(data){
          $ionicLoading.hide();  
          if(data.type == "OK"){
            if ($scope.nextCommentPage > 1) {
              $scope.comments = $scope.comments.concat(data.comments.result);
            }else{
              $scope.comments =  data.comments.result;
            }
            
            $scope.nextCommentPage = data.comments.next;
            $scope.$broadcast('scroll.refreshComplete');
          }
        });
    };


    $scope.closeComments = function(){
      $scope.commentsModal.hide();
      $scope.comments = {};
      $scope.nextCommentPage = 1;
    };

   $scope.newComment = {};

   //adding new comment
   $scope.addComment = function(){

     $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Posting...'
     });

     CommentCrud.create($scope.social_id,$scope.newComment,function(data){
        $ionicLoading.hide();  
        if(data.type == "OK"){
          $scope.comments.unshift(data.comment);
          $scope.newComment = {};
          //for increment in event comment count
          $scope.channel.socials.comments.push(data.comment);
        }
      });
    };


    //remove comment
    $scope.removeComment = function(comment){
      var confirmPopup = $ionicPopup.confirm({
         title: 'Delete Comment',
         template: 'You are about to delete the comment ?'
       });
       //delete circle if user click okay
       confirmPopup.then(function(res) {
          if (res) {
              var index = $scope.comments.indexOf(comment);
              $scope.comments.splice(index,1);
              CommentCrud.delete($scope.social_id,comment.id,function(data){
                if (data.type == "OK") {
                  $scope.message = '';
                  $scope.event.socials.comments.splice(0,1);
                }else{
                  $scope.message = data.message;
                }
            });

          }
       });
    };



   $scope.isMe = function(id){
    return Contacts.isMe(id);
   };
     
   //load moments 
    //get all moment related to this channel
  $scope.getMoments = function(useCache){    
    var loading = $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading moments...'
    });
    //calling the exact same method for subscribing channel
    //yet with different params 
    MomentCrud.getMomentsByChannelID($scope.channel.id, useCache, function(data){
      if (data.type == "OK") {
       $ionicLoading.hide();
       $scope.moments = data.moments;
      }else{
        $scope.message = data.message;
      }
    });
  };

  //view deatil of moment
  $scope.viewPhoto = function(moment){
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading...'
    });

    SocialCrud.detail(moment.social_id,function(data){
      $ionicLoading.hide(); 
      if(data.type == "OK"){
        $scope.moment = data.social;
        $state.go('menu.tab.account-photo',{"moment" : data.social, "account" : $scope.account});
      }else{
        $scope.message = data.message;
      }
    });
   
  };

   //update the channel 
  //also upate the local data by giving 
  //type (created|joined) , index and channel
  $scope.loadChanel = function(useCache) {
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading channel...'
    });

    ChannelCrud.detail($scope.channel,function(data){
      $ionicLoading.hide();
      $scope.$broadcast('scroll.refreshComplete');
      
      if (data.type == "OK") {

        $scope.message = '';

        $scope.channel = data.channel;

        $scope.index = Channels.findChannelIndex( $scope.channel, $rootScope.channels.joined);  
        
        var channel = angular.copy($scope.channel);
        
        channel.photo = $scope.channel.newPhoto;

        $scope.getMoments(useCache);
      
      }else{
        $scope.message = data.message;
      }
    });
  };

  $scope.loadChanel(false);


  //comments modal template
  $ionicModal.fromTemplateUrl('templates/modal-view-comments.html', function($ionicModal) {
    $scope.commentsModal = $ionicModal;
  }, {
    scope: $scope,
    focusFirstInput: true,
    animation: 'slide-in-up'
  });

  $ionicPopover.fromTemplateUrl('templates/popover-channel-detail-setting.html', {
    scope: $scope
  }).then(function(popover) {
    $scope.settingPopover = popover;
  });

});
