angular.module('gurillaApp.services')

.factory('Circles', function(LocalData,CircleCrud) {

  return {
    init:function(callback){
      CircleCrud.read(function(data){
          LocalData.set('circles', JSON.stringify(data.circles));
          callback(data.circles);
      },function(data){
        console.error(data);
      });
    },
    initGurillaContacts: function(strcontacts){
     var contacts = [];
     if (strcontacts) {
      contacts = JSON.parse(strcontacts);
     }else{
      var account = LocalData.getObject('account');
      contacts = JSON.parse(account.contacts);
     }

     var gcontacts = [];
     for (var i = contacts.length - 1; i >= 0; i--) {
       if(contacts[i].gurilla){
        gcontacts.push(contacts[i].id);
       }
     }
     LocalData.set('gurilla_contacts',JSON.stringify(gcontacts));
     return gcontacts;
    },
    all: function() {
      var circlesData = LocalData.get('circles');
      var circles = undefined;
      if (typeof circlesData !== 'undefined') {
        circles = JSON.parse(circlesData);
      }
      return circles;
    },
    getContacts: function(id) {
      var circlesData = LocalData.get('circles');
      var circles = null;
      if (typeof circlesData !== 'undefined') {
        circles = JSON.parse(circlesData);
      }
      if (typeof circles[id] !== 'undefined') {
        return circles[id].contacts;
      } else {
        return null;
      }
    },
    set : function(circles){
      console.log("Circle",circles);
      for (var i = 0; i < circles.length; i++) {
        if (circles[i].contacts && circles[i].contacts.length > 0) {
          circles[i].contacts = JSON.parse(circles[i].contacts);
        }
      }
      console.info("set new circle data",LocalData.setObject('circles', circles));
      return circles;
    },
   
    update: function(index,circle) {

      var data = LocalData.get('circles');
      var circles = undefined;

      if (typeof data !== 'undefined') {
        circles = JSON.parse(LocalData.get('circles'));
      }

      if (circle.contacts.length > 0 ) {
        circle.contacts = JSON.parse(circle.contacts);
      }

      circles[index] = circle;

      LocalData.set('circles', JSON.stringify(circles));

      return circles;
    },
    get: function(id) {
      var circlesData = LocalData.get('circles');
      var circles = null;
      if (typeof circlesData !== 'undefined') {
        circles = JSON.parse(circlesData);
      }
      if (typeof circles[id] !== 'undefined') {
        return circles[id];
      } else {
        return null;
      }
    },
    add: function(circle) {
      var data = LocalData.get('circles');
      var circles = undefined;
      if (typeof data !== 'undefined') {
        circles = JSON.parse(data);
      }
      if (circle.contacts.length > 0 ) {
        circle.contacts = JSON.parse(circle.contacts);
        
      }

      circles.push(circle);
      
      LocalData.set('circles', JSON.stringify(circles));
      return circles;
    },
    remove: function(index) {
      var data = LocalData.get('circles');
      var circles = undefined;
      
      if (typeof data !== 'undefined') {
        circles = JSON.parse(LocalData.get('circles'));
      }

      circles.splice(index,1);

      LocalData.set('circles', JSON.stringify(circles));

      return circles;
    },
    getCameraOptions : function(source){
      var options = {
          quality: 70,
          sourceType: source,
          encodingType: navigator.camera.EncodingType.JPEG,
          destinationType: navigator.camera.DestinationType.DATA_URL,
          correctOrientation: true,
          targetWidth: 200,
          targetHeight : 200,
          mediaType : navigator.camera.MediaType.PICTURE,
          allowEdit: true,
          saveToPhotoAlbum: false
        };

      return options; 
    },
    //sync the circle with new Gurilla Contacts
    //if found set gurilla to true
    syncCircleContacts : function(newGContacts){
      // get local circles data
      var circles = JSON.parse(LocalData.get('circles'));
      // var oldCircleContacts = JSON.parse(circles.contacts);
      
      for (var i = circles.length - 1; i >= 0; i--) {
       var c = angular.copy(circles[i].contacts);
        for (var j = c.length - 1; j >= 0; j--) {
          console.debug(c[j].displayName+" id is " + c[j].id);
          for (var k = newGContacts.length - 1; k >= 0; k--) {
            if (c[j].id == newGContacts[k].id) {
              console.debug(c[j].displayName , true);
              c[j].gurilla = true;
            }else{
              if (c[j].gurilla) {
                console.debug(c[j].displayName + " is being set false !!!!");
              }
            }
            
          }
          console.debug("Final contact for circle - " +circles[i].name ,c);
        }
    
        circles[i].contacts = JSON.stringify(c);
          console.log("\n");
      }
      this.set(circles);
      return circles;
    }
  };
})
.factory('CircleCrud', ['$http','$rootScope','GURILLA_URL','LocalData','MESSAGE',function ($http,$rootScope,GURILLA_URL,LocalData,MESSAGE) {
  var url = {
    "read" : GURILLA_URL +'circles/user',
    "create" : GURILLA_URL +"circles",
    "update" : GURILLA_URL +"circles",
    "delete" : GURILLA_URL + 'circles/delete'
  };

  var payload = { "token" : LocalData.getToken() };

  var config = { cache: true };

  var getResponse = function(){
      return {"type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  return {
      
      read : function(callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          $http.post( url.read , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      create : function(circle,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          payload.name = circle.name;
          payload.photo = circle.photo;
          payload.contacts = circle.contacts;

          $http.post( url.create, payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      update : function(circle , callback){

        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          payload.name = circle.name;
          payload.contacts = circle.contacts;
          payload.photo = circle.newPhoto;

          $http.put( url.update+'/'+circle.id , payload ,config).success(function(data, status) {
            if (status == 200) {
              response = data;
            }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 


      },
      delete : function(id,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          $http.post( url.delete+'/'+id , payload ,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      }

  };

}]);
