angular.module('gurillaApp.services')

.factory('Contacts', function($http, $rootScope, $ionicLoading, LocalData, GURILLA_URL,User) {
  return {
    lookCheckKey : function(haystack, needle){
      //put check attr to true if there are contacts previously selected
      if (needle.length > 0) {
        //check for all previous selected contacts 
        //in fresh contacts
        for (var i = 0; i < needle.length; i++) {
          for (var j = 0; j < haystack.length; j++) {
            angular.forEach(needle[i].phoneNumbers, function(p1){
              angular.forEach(haystack[j].phoneNumbers, function(p2){
                  if(p1 == p2){
                    haystack[j].checked = true;
                  }
              });
            });
           }
        }
      }
      return haystack;
    },
    isMe : function(id){
      var user = JSON.parse(LocalData.get('account'));
      return user.id == id;
    },
    isMeWithPhone : function(phone){
      var user = JSON.parse(LocalData.get('account'));
      return user.phone == phone;
    },
    getAll: function() { /* get all contacts including non-Gurilla */
      var cstring = LocalData.get('local_contacts');
      if (typeof cstring != 'undefined') {
        return JSON.parse(cstring);
      } else {
        return [];
      }
    },
    getGurilla: function() { /* get contacts that signed up with Gurilla */
      var user = JSON.parse(LocalData.get('account'));
      if (user.contacts) {
        return JSON.parse(user.contacts);
      } else {
        return [];
      }
    },
    getCircle: function(id) {
      var cstring = LocalData.get('circle_contacts_'+id);
      var contacts = null;
      if (typeof cstring != 'undefined') {
        contacts = JSON.parse(cstring);
      }
      return contacts;
    },
    download: function() { /* download contacts from server to local - should be used only in browser */
      /* Browser, downloading contacts */
      var token = LocalData.getToken();
      var config = { cache: false };
      $http.post(
        GURILLA_URL + 'users/contacts',{'token': token},config).success(function(data, status) {
        if (data.type == 'OK') {
          LocalData.set('gurilla_contacts', data.contacts);
        } else {
          LocalData.set('gurilla_contacts_error', data.message);
        }
      }).error(function(data, status){
        if (status == 0) {
          console.error('Unable to send data. No network connection?');
        } else {
          if (data.code == 403) { /* server said we are forbidden */
            LocalData.clear();
            window.location.href = 'index.html#/login';
          }
        }
      }).finally(function(){
         $rootScope.$emit('finish loading');
      });
    },
    //depreciate
    load : function(){
      var user = JSON.parse(LocalData.get('account'));
      LocalData.set('gurilla_contacts', user.contacts);
    },
    sync : function(){
      User.sync(function(data){
        console.log('sync finshed');
      });
    }
  };
})

.service("ContactsService", ['$q', function($q) {
  var formatContact = function(contact) {
    return {
        "displayName"   : contact.name.formatted || contact.name.givenName + " " + contact.name.familyName || "Mystery Person",
        "emails"        : contact.emails || [],
        "phones"        : contact.phoneNumbers || [],
        "photos"        : contact.photos || []
    };
  };

  var pickContact = function() {
    var deferred = $q.defer();
    if(navigator && navigator.contacts) {
      navigator.contacts.pickContact(function(contact){
        deferred.resolve( formatContact(contact) );
      });
    } else {
      deferred.reject("Bummer.  No contacts in desktop browser");
    }
    return deferred.promise;
  };

  return {
    pickContact : pickContact
  };
}]);