var app = angular.module('gurillaApp.controllers');
app.constant('TB_EVENTS', {'CREATED' : 'created','INVITED' : 'invited','PENDING' : 'pending'});
app.constant('NB_EVENTS', { '1' : 'created','2' : 'invited','3' : 'pending'});
app.constant('NNB_EVENTS',{'CREATED' : 1,'INVITED' : 2,'PENDING' : 3});
app.constant('MSGTYPE', { 'CREATED' : 1 , 'EDITED': 2,'CANCELLED': 3  });
app.constant('STATUS' , {'ACTIVE' : 1 , 'INACTIVE' : 0});

// Event Controller
app.controller('EventsCtrl', function (
    $rootScope, $scope, $state, $stateParams, $http, $ionicHistory, nzTour, $timeout,
    $ionicModal, $ionicLoading, $ionicPopup,LocalData, Events, EventCrud, Circles, GURILLA_URL , TB_EVENTS ,NB_EVENTS, CalendarCrud, Helper, Tutorials) {
    $rootScope.message = '';


   Helper.showTutorial(Tutorials.event);

   $scope.createNewEvent = function(){
      if(Helper.isTutorialFinished(Tutorials.event)){
        $state.go("menu.tab.event-create");
      }
    };


    $scope.activeButton = LocalData.get('activeButton');
    if ($scope.activeButton == undefined) {
      $scope.activeButton = 1;
    }

    $scope.$watch('activeButton', function(newVal,oldVal){
      $scope.events = Events.GetEventsByType(NB_EVENTS[$scope.activeButton]);
      LocalData.set('activeButton',$scope.activeButton);
      
    });

    $scope.changeActiveEvents = function(number){
      if(Helper.isTutorialFinished(Tutorials.event)){
        $scope.activeButton = number;  
      }
    };

    //Refresh function which is going to fetch all the events from server 
    //Update the local data as well
    //Control over the show and hide loading icon
    $scope.tryRefresh = function(callback){
      //fetch the data from server
    
      EventCrud.read(function(data){
        $scope.$broadcast('scroll.refreshComplete');

        if (data.type == "OK") {
          $scope.message = '';
          //create new buffer data
          var e = { created: [], invited: [],pending:[] };
          //fill with coresponding data return by server
          e.created = data.my_events;
          e.invited = data.events.confirmed;
          e.pending = data.events.pending;
          //update local data as well as global channels
          callback(Events.set(e));
          
        }else{
          $scope.message = data.message;
        }
      });
    };

    $scope.refresh = function(){
      $scope.tryRefresh(function(data){
        $scope.events = Events.GetEventsByType(NB_EVENTS[$scope.activeButton]);
        $scope.ehistory = Events.updateHistory(data);
      });
    };

    $scope.isHide = function(event){
      if (event && $scope.ehistory[event.id]) {
        return $scope.ehistory[event.id].hide | false;
      }
    };

    $scope.isExpired = function(event){
      if (event) {
        return new Date(event.date + " " + event.time) <= new Date();
      }
    };

    $scope.isNewer = function(event){
      if (event && $scope.ehistory[event.id]) {
        return event.version >  parseInt($scope.ehistory[event.id].version);
      }
    };

    $scope.isCancelled = function(event){
      if ( event && $scope.ehistory[event.id] ) {
        return event.event_status == 0 || $scope.ehistory[event.id].active == 0;
      }
    };

    $scope.isTouch = function(event){
      if (event && $scope.ehistory[event.id]) {
        return $scope.ehistory[event.id].touched;
      }
    };

    $scope.isEmptyEventLists = function(list){
      if (list.length == 0) return true;
      var hidecount = 0;
      for (var i = 0; i < list.length; i++) {
        if ($scope.isHide(list[i])) {
          hidecount++;
        }
      }
      return list.length == hidecount;
    };

    $scope.getUnTouchedCount = function(number){

      var event = Events.GetEventsByType(NB_EVENTS[number]);
      if (event) {
        var untouched_count = 0;
        for (var i = 0; i < event.length; i++) {
          if($scope.ehistory[event[i].id] && !$scope.ehistory[event[i].id].touched){
            untouched_count++;
          }
        }
        return untouched_count;
      }
      return 0;
     
    };

    $scope.touch = function(event){
      if (!$scope.ehistory[event.id].touched) {
        $scope.ehistory = Events.updateHistoryNode(event);
      }
    };

     //User cancel the created event which not expired yet
    $scope.cancel = function(event){
       var confirmPopup = $ionicPopup.confirm({
         title: 'Cancel Event',
         template: 'Are you sure you want to cancel this event?'
       });


       confirmPopup.then(function(res) {
        if (res) {

            //remove the event from created list
            Events.remove(NB_EVENTS[$scope.activeButton],event,function(data){
              $scope.events = data;
              //remove related history from localdb
              Events.deleteHistoryNode(event);
            });

            //update the status on server
            EventCrud.cancel(event.id,function(data){
              if (data.type == "OK") {
                console.log('Cancelled');
                // delete event in local calendar
                CalendarCrud.delete(event);
              }
            });
        }
      });

    };
    //user delete expired event
    $scope.delete = function(event) {
     var confirmPopup = $ionicPopup.confirm({
       title: 'Delete Event',
       template: 'Are you sure you want to delete this event?'
     });

     confirmPopup.then(function(res) {
       if (res) {
        //Remove event from localdb

        //Delete event from server
        EventCrud.delete(event.id,function(data){
          if (data.type == "OK") {
              CalendarCrud.delete(event);
              Events.remove(NB_EVENTS[$scope.activeButton],event,function(data){
                $scope.events = data;
                //remove related history from localdb
                Events.deleteHistoryNode(event);
                
              });
          }else{

          }
         });
       }
     });
    };

    //detail view
    $scope.detail = function(event){
        $scope.touch(event);
        $state.go('menu.tab.event-detail',{'id' : event.id});
    };
    //edit view
    $scope.edit = function(event){
      $state.go('menu.tab.event-manage',{'event':event});
    };


    $scope.openCalendar = function() {
      if(Helper.isTutorialFinished(Tutorials.event)){
        $scope.message = CalendarCrud.open();
      }
    };

    //check the global events object for undefined
    if (angular.isUndefined($scope.events)) {
      //retrieve the data from local data
      $scope.events = Events.GetEventsByType(TB_EVENTS.CREATED);
    
      //still undefined
      if(angular.isUndefined($scope.events)){
        $ionicLoading.show({
            template: '<i class="icon ion-loading-c"></i> Loading events...'
        });
        // //fetch the data from server
        $scope.tryRefresh(function(data){
          $ionicLoading.hide();
          $scope.events = data[TB_EVENTS.CREATED];
          $scope.ehistory = Events.writeHistory(data);
        });
      }else{
          $scope.ehistory = LocalData.forceReadObject('ehistory');
      }
    }

    $timeout(function(){
      //juz for workaround
      $scope.refresh();
    }, 1000);
    
});

// // Edit event controller 
app.controller('EventManageCtrl', function (
$rootScope,$filter, $scope, $state, $stateParams, $http, $ionicHistory,
$ionicModal, $ionicLoading, $ionicPopup, $ionicActionSheet,EventCrud,$cordovaCamera,
LocalData, Events,Interests,Circles,Channels, GURILLA_URL , TB_EVENTS ,NB_EVENTS, CalendarCrud) {
   
   $scope.newPhotos = [];
  

   
   //get the index from global events 
   $scope.event = $stateParams.event;

   $scope.event.quantity = parseInt($scope.event.quantity);
  
   $scope.oldEvent = JSON.parse(JSON.stringify($scope.event));

    //show modal 
    $scope.editPhoto = function(){
      $scope.copyOfPhotos = angular.copy($scope.event.photos);
      $scope.modalPhoto.show();
    };
    
    //cancle photo remove 
    //revert the changes to orignal 
    $scope.cancelPhotoEdit = function() {
        $scope.modalPhoto.hide();
    };
    //remove the selected photo
    $scope.removePhoto = function($index){
      $scope.copyOfPhotos.splice($index,1);
    };
    
    $scope.removeNewPhoto = function($index){
        $scope.newPhotos.splice($index,1);
    };
    //remove the photo from actual array
    //reset the copy to blank
    $scope.donePhotoEdit = function(){
      $scope.modalPhoto.hide();
      $scope.event.photos = $scope.copyOfPhotos;
      $scope.copyOfPhotos = '';
      console.info($scope.event);
    };
  

    //Comment about
    $scope.showAboutModal = function() {
        $scope.modalAbout.show();
        $rootScope.hideKeyboardAccessoryBar(true);
    };
    //
    $scope.updateAbout = function() {
        $scope.modalAbout.hide();
        $rootScope.hideKeyboardAccessoryBar(false);
    };

    $scope.isSelected = function(haystack,needle){
      for (var i = 0; i < haystack.length; i++) {
        if(needle.id == haystack[i].id){
          return i;
        }
      }
    };

    //show model to choose interests
    $scope.showModal = function(str){
    
     if ( angular.isUndefined($scope.tmpstr) || str != $scope.tmpstr ) {
       $scope.tmpstr = str;
       var tmpdata = [];
       if (str == 'channels') {
          angular.copy($rootScope[str].created, tmpdata);
       }else{
          angular.copy($rootScope[str], tmpdata);
       }

       $scope.tmpdata = tmpdata;

       for (var i = 0; i < $scope.event[str].length; i++) {
          $index = $scope.isSelected(tmpdata,$scope.event[str][i]);
          $scope.tmpdata[$index].checked = true;
       }
 
     }else{
      console.log("again");
     }

     $scope.modalTempData.show();
    };    

    //Update event interests 
    $scope.updateModal = function(str){
      var objects = [];
      angular.forEach($scope.tmpdata, function(obj,index){
        if (obj.checked) {
          objects.push(obj);
        }
      });
      $scope.event[$scope.tmpstr] = objects; 
      $scope.modalTempData.hide();
    };

    $scope.showDate = function() {
      if (typeof datePicker !== 'undefined') {
        var today = new Date();
        var cdate = new Date();
        if ($scope.event.date != '') {
          cdate = new Date($scope.event.date);
          if (cdate == 'Invalid Date') {
            cdate = new Date();
          }
        }
        var options = {
          date: cdate,
          minDate: today.toDateString(),
          allowOldDates: false,
          allowFutureDates: true,
          mode: 'date'
        };

        datePicker.show(
          options,
          function(date) {
            if (date == 'Invalid Date') return;
            $scope.event.date = date.toDateString();
            document.getElementById('date').value = $scope.event.date;
          }
        );
      } else {
        $scope.message = 'Date Picker is not available. Are you on browser?';
      }
    };

    $scope.showTime = function() {
        if (typeof datePicker !== 'undefined') {
            var today = new Date();
            var ctime = new Date($scope.event.date);
            if ($scope.event.time != '') {
                if(typeof $scope.event.time_in_ms !== 'undefined'){
                    ctime.setTime($scope.event.time_in_ms);
                }else{
                    ctime.setHours($scope.event.time.substr(0,2));
                    ctime.setMinutes($scope.event.time.substr(3,2));
                }
            }
            if (ctime == 'Invalid Date'){
                ctime = new Date($scope.event.date);
            }
            var options = {
              date: ctime,
              minDate: today,
              minuteStep: 1,
              mode: 'time'
            };
            datePicker.show(
              options,
               function(dateTime) {
                if (dateTime == 'Invalid Date') return;
                dateTime.setSeconds(0);
                var realDateTime = new Date(dateTime);
                realDateTime = realDateTime.toTimeString();
                realDateTime = realDateTime.substring(0,realDateTime.indexOf("GMT"));
                $scope.event.time = realDateTime;
             //   $scope.event.time_formated = $scope.event.time.substr(0,5);
                $scope.event.time_in_ms = dateTime.getTime();
                document.getElementById('time').value = $scope.event.time;
              }
            );

        } else {
            $scope.message = 'Time Picker is not available. Are you on browser?';
        }
    };  

    $scope.openCameraActionSheet = function(){
        var hideSheet = $ionicActionSheet.show({
          buttons: [
            { text: 'Use Camera' },
            { text: 'Choose Photo' }
          ],
          cancelText: 'Cancel',
          cancel: function() {},
          buttonClicked: function(index) {
            var source = 0;
            if (navigator.camera) {
              if (index == 0) {
                source = navigator.camera.PictureSourceType.CAMERA;
              } else if (index == 1) {
                source = navigator.camera.PictureSourceType.PHOTOLIBRARY || 0;
              }

              var options = Events.getCameraOptions(source);

              $cordovaCamera.getPicture(options).then(function(imageURI) {
                var photo = { id : 0,  event_id : $scope.event.id , image : imageURI };
                $scope.newPhotos.push(photo);

              }, function(err) {
                console.log(err);
              });

              // navigator.camera.getPicture(
              //   function(imageURI) {
              //     var photo = { id : 0,  event_id : $scope.event.id , image : imageURI };
              //     $scope.newPhotos.push(photo);
              //     $scope.$apply();
              //   }, function(err) {
              //     console.err("Error");
              //   }, options);
            } else {
              $rootScope.message = 'Unable to access camera. Are you on browser?';
            }

            return true;
      }
    });
    };
    
    
    $scope.tryUpdate = function(){
      
      $ionicLoading.show({
          template: '<i class="icon ion-loading-c"></i> Updating event..'
      });

      var edit_event = angular.copy($scope.event);

      edit_event.photos = edit_event.photos.concat($scope.newPhotos);

      EventCrud.update(edit_event,function(data){
        if (data.type == "OK") {
         
         $ionicLoading.hide();

         data.event.accept_count = $scope.event.accept_count;

         Events.addHistoryNode(data.event);

         Events.update(TB_EVENTS.CREATED,data.event);

         $state.go('menu.tab.events');
         console.log('done updating before updating calendar');
         // delete current event inside calendar and add in new event's details
         CalendarCrud.update($scope.oldEvent,$scope.event);
         
        }else{
          console.log(data);
        }

      });

      
    };

    // about modal
    $ionicModal.fromTemplateUrl('templates/modal-newEvent-about.html', function($ionicModal) {
        $scope.modalAbout = $ionicModal;
    }, {
        scope: $scope,
        focusFirstInput: true,
        animation: 'slide-in-up'
    });
    
   // temp modal
    $ionicModal.fromTemplateUrl('templates/modal-event-tempdata.html', function($ionicModal) {
        $scope.modalTempData = $ionicModal;
    }, {
        scope: $scope,
        focusFirstInput: true,
        animation: 'slide-in-up'
    });

    //
    //Photo editing modal
    //can remove photos
    //
    $ionicModal.fromTemplateUrl('templates/modal-event-photo.html', function($ionicModal) {
        $scope.modalPhoto = $ionicModal;
    }, { 
        scope: $scope,
        focusFirstInput: false,
        animation: 'slide-in-up'
    });
});

// Create event controller
app.controller('EventCreateCtrl', function ($cordovaCamera,
    $rootScope,$scope,$state,$ionicModal,$ionicLoading,Events,$injector,$http,$ionicActionSheet,$ionicPopup,$window,
    GURILLA_URL,LocalData,$stateParams,TB_EVENTS ,NNB_EVENTS,NB_EVENTS,EventCrud,CalendarCrud,STATUS, User) {
    
    
    var done = false;
    $rootScope.message = '';
    
    
    //new event 
    $scope.event = {
      id : 0,
      event_status : STATUS.ACTIVE,
      version : 0,
      interests : [],
      channels : [],
      contacts : [],
      circles : [],
      photos : []
    };
    
    //Cancel creating event
    $scope.cancelCreating = function() {
        $state.go('menu.tab.events');
    };

    //Comment about
    $scope.showAboutModal = function() {
        $scope.modalAbout.show();
        $rootScope.hideKeyboardAccessoryBar(true);
    };
    //
    $scope.updateAbout = function() {
        $scope.modalAbout.hide();
        $rootScope.hideKeyboardAccessoryBar(false);
    };

    //show modal 
    $scope.editPhoto = function(){
      $scope.copyOfPhotos = $scope.event.photos.slice(0);
      $scope.modalPhoto.show();
    };
    
    //cancle photo remove 
    //revert the changes to orignal 
    $scope.cancelPhotoEdit = function() {
        $scope.modalPhoto.hide();
    };
    //remove the selected photo
    $scope.removePhoto = function($index){
      $scope.copyOfPhotos.splice($index,1);
    };
    //remove the photo from actual array
    //reset the copy to blank
    $scope.donePhotoEdit = function(){
      $scope.modalPhoto.hide();
      $scope.event.photos = $scope.copyOfPhotos;
      $scope.copyOfPhotos = {};

    };


    var tempObjects = {
      "interests" : [],
      "channels" : [],
      "circles" : []
    };

    if ($stateParams.object !== null ) {
      $scope.event[$stateParams.type] = [angular.copy($stateParams.object)];
    }

    //show model to choose interests
    $scope.showModal = function(str){
     
     if ( angular.isUndefined($scope.tmpstr) || str != $scope.tmpstr ) {
       var tmpdata = [];
       
       if (tempObjects[str].length > 0) {
          tmpdata = tempObjects[str];
       }else{
         if (str == 'channels') {
            angular.copy($rootScope[str].created, tmpdata);
         }else{
            angular.copy($rootScope[str], tmpdata);
         }
       }
       $scope.tmpdata = tmpdata;
       $scope.tmpstr = str;
       if ($scope.tmpstr == $stateParams.type && !done) {
         done = true;
         for (var i = 0; i < $scope.tmpdata.length; i++) {
          if ($stateParams.object.id == $scope.tmpdata[i].id){
           $scope.tmpdata[i].checked = true;
          } 
         }
       }  
     }

     $scope.modalTempData.show();
    };
    //get all of the ph as comma seperated string 
    //from a given event objecty
    
    $scope.getNonGurillaAcc = function(obj){
      var phones = '';
      for (var i = 0; i < obj.circles.length; i++) {
        var contacts = JSON.parse(obj.circles[i].contacts);
          for (var j = 0; j < contacts.length; j++) {
            if (!User.isGurillaUser(contacts[j])) {
              phones = ( phones.length > 0)? 
                  phones+','+contacts[j].phoneNumbers[0] : contacts[j].phoneNumbers[0] ;
            }
        }
      }
      return phones;
    };
    
    $scope.sendMessage = function(eventObj,strPhones){  
      if ($window.plugins) {
        var subject = "You've Been Invited!\n\nEvent Name: "+eventObj.title+"\nDate: "+eventObj.date+"\nTime: "+eventObj.time+"\nVenue: "+eventObj.venue+"\n\nCheck out Gurilla App to create and manage your own events, or start your own movement!"+"\n http://goo.gl/yWSlC6";
        $window.plugins.socialsharing.shareViaSMS(subject,strPhones,function(msg) {
            console.log('ok: ' + msg);
          },function(msg) {
            alert('error: ' + msg);
          });
      }
    };

     //Update event interests 
    $scope.updateModal = function(){
      var item = [];
      tempObjects[$scope.tmpstr] = $scope.tmpdata;
      angular.forEach($scope.tmpdata, function(obj,index){
        if (obj.checked) {
          if($scope.tmpstr == 'channels'){
            item.push($rootScope[$scope.tmpstr].created[index]);
          }else{
            item.push($rootScope[$scope.tmpstr][index]);
          }
        }
      });

      $scope.event[$scope.tmpstr] = item;
      $scope.modalTempData.hide();
    };

    $scope.showDate = function() {
      if (typeof datePicker !== 'undefined') {
        var today = new Date();
        var cdate = new Date();
        if ($scope.event.date != '') {
          cdate = new Date($scope.event.date);
          if (cdate == 'Invalid Date') {
            cdate = new Date();
          }
        }
        var options = {
          date: cdate,
          minDate: today.toDateString(),
          allowOldDates: false,
          allowFutureDates: true,
          mode: 'date'
        };

        datePicker.show(
          options,
          function(date) {
            if (date == 'Invalid Date') return;
            $scope.event.date = date.toDateString();
            document.getElementById('date').value = $scope.event.date;
          }
        );
      } else {
        $scope.message = 'Date Picker is not available. Are you on browser?';
      }
    };

    $scope.showTime = function() {
        if (typeof datePicker !== 'undefined') {
            var today = new Date();
            var ctime = new Date();
            if (typeof $scope.event.time !== 'undefined' && $scope.event.time != '') {
              ctime.setTime($scope.event.time_in_ms);
            }
            var options = {
              date: ctime,
              minDate: today,
              minuteStep: 1,
              mode: 'time'
            };

            datePicker.show(
              options,
               function(dateTime) {
                if (dateTime == 'Invalid Date') return;
          //      $scope.event.time = dateTime.toTimeString();
                $scope.event.time_in_ms = dateTime.getTime();
                dateTime.setSeconds(0);
                var realDateTime = new Date(dateTime);
                realDateTime = realDateTime.toTimeString();
                realDateTime = realDateTime.substring(0,realDateTime.indexOf("GMT"));
                $scope.event.time = realDateTime;
               // $scope.event.time_formated = $scope.event.time.substr(0,5);
                document.getElementById('time').value = $scope.event.time;
              }
            );

        } else {
            $scope.message = 'Time Picker is not available. Are you on browser?';
        }
    };

    $scope.openCameraActionSheet = function(){
        var hideSheet = $ionicActionSheet.show({
          buttons: [
            { text: ' Use Camera' },
            { text: ' Choose Photo' }
          ],
          cancelText: 'Cancel',
          cancel: function() {
          },
          buttonClicked: function(index) {
            var source = 0;

        if (navigator.camera) {
          if (index == 0) {
            source = navigator.camera.PictureSourceType.CAMERA;
          } else if (index == 1) {
            source = navigator.camera.PictureSourceType.PHOTOLIBRARY || 0;
          }

          var options = Events.getCameraOptions(source);

          $cordovaCamera.getPicture(options).then(function(imageURI) {
            var photo = { id : 0,  event_id : $scope.event.id , image : imageURI };
            $scope.event.photos.push(imageURI);
          }, function(err) {
            console.log(err);
          });

    
          // navigator.camera.getPicture(

          //   function(imageURI) {
          //     $scope.event.photos.push(imageURI);
          //     $scope.$apply();
          //   }, function(err) {
          //     console.err(err);
          //   },
          //   Events.getCameraOptions(source));
        } else {
          $rootScope.message = 'Unable to access camera. Are you on browser?';
        }

        return true;
      }
    });
    };

   

    $scope.tryCreate = function(){
      $ionicLoading.show({
          template: '<i class="icon ion-loading-c"></i> Creating event..'
      });

      $scope.event.reminder = $scope.event.reminder | 15;

      console.log($scope.event);
      
      EventCrud.create($scope.event,function(data){
        $ionicLoading.hide();
        if (data.type == "OK") {
          $scope.event.id = data.event.id;
          //add event localstorage events obj 
          Events.add(TB_EVENTS.CREATED, data.event);
          //add history
          Events.addHistoryNode(data.event);

          var strPhones = $scope.getNonGurillaAcc(data.event);
          if (strPhones.length > 0) {
            var confirmPopup = $ionicPopup.confirm({
             title: 'Send Message',
             template: 'Would you like to send message via SMS to non Gurilla contacts? (Local charges apply)'
            });

            confirmPopup.then(function(res) {
             if (res) {
               $scope.sendMessage(data.event,strPhones);
             }
            });
          }

          //Create event in local calendar
          CalendarCrud.create($scope.event);

          $state.go($rootScope.prevState,$rootScope.prevParams);
          // $state.go('menu.tab.events');
          LocalData.set('activeButton',NNB_EVENTS.CREATED);
        }else{
          $rootScope.message = data.message;
        }
      });
    };

     // about modal
    $ionicModal.fromTemplateUrl('templates/modal-newEvent-about.html', function($ionicModal) {
        $scope.modalAbout = $ionicModal;
    }, {
        scope: $scope,
        focusFirstInput: false,
        animation: 'slide-in-up'
    });

    // channels modal
    $ionicModal.fromTemplateUrl('templates/modal-event-tempdata.html', function($ionicModal) {
        $scope.modalTempData = $ionicModal;
    }, {
        scope: $scope,
        focusFirstInput: true,
        animation: 'slide-in-up'
    });

    //
    //Photo editing modal
    //can remove photos
    //
    $ionicModal.fromTemplateUrl('templates/modal-newEvent-photo.html', function($ionicModal) {
        $scope.modalPhoto = $ionicModal;
    }, {
        scope: $scope,
        focusFirstInput: false,
        animation: 'slide-in-up'
    });

});

//
// Event detail controller
//
app.controller('EventDetailCtrl', function(
    $window,$scope, $state, $stateParams,$ionicLoading, $ionicPopup, LocalData, Events,MSGTYPE,$ionicHistory,
     EventCrud,TB_EVENTS,NNB_EVENTS ,NB_EVENTS ,Contacts,$ionicModal, CommentCrud ,CalendarCrud,STATUS) {

    $scope.para = $stateParams.data;
    // get the event from global events
    $scope.event = {};
    $scope.pending = false;

    $scope.activeButton = LocalData.get('activeButton');

    $scope.getAcceptCount = function(event){
      if(!event.accept_count){
        event.accept_count = 0;   
      }
      return event.accept_count;
    };

    $scope.isFull = function(event){
      if (event) {
        return event.accept_count == event.quantity;
      }
    };
    
    $scope.viewComings = function(event){
        
        $scope.comingUsersModal.show();

        if (event.accept_count > 0) {
          $ionicLoading.show({
            template: '<i class="icon ion-loading-c"></i> Loading users...'
          });

          EventCrud.getComings(event.id,function(data){
            $ionicLoading.hide();
            if (data.type == "OK") {
              $scope.users = data.users;
            }
          });  
        }else{
          $scope.users = [];
        }
    };

    $scope.isMe = function(id){
      return Contacts.isMe(id);
    };


    $scope.newComment = {
      "content" : ""
    };

    $scope.social_id = -1;
    
    $scope.nextCommentPage = 1;
    $scope.viewComments = function(isRefresh){

      if(isRefresh){
        $scope.nextCommentPage = 1;
      }

      if ($scope.event.socials) {
        $scope.social_id = $scope.event.socials.id;
      }

      if (!$scope.commentsModal.isShown()) {
          $scope.commentsModal.show();
      }
      
      if ( $scope.social_id < 1) {
        return;
       }

       $ionicLoading.show({
          template: '<i class="icon ion-loading-c"></i> Loading comments...'
       });
       
        CommentCrud.read($scope.social_id,$scope.nextCommentPage,function(data){
          $ionicLoading.hide();  
          if(data.type == "OK"){
            if ($scope.nextCommentPage > 1) {
              $scope.comments = $scope.comments.concat(data.comments.result);
            }else{
              $scope.comments =  data.comments.result;
            }
            
            $scope.nextCommentPage = data.comments.next;
            $scope.$broadcast('scroll.refreshComplete');
          }
        });
    };


    $scope.closeComments = function(){
      $scope.commentsModal.hide();
      $scope.comments = {};
      $scope.nextCommentPage = 1;
    };


    $scope.addComment = function(){

     $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Posting...'
     });

     CommentCrud.create($scope.social_id,$scope.newComment,function(data){
        $ionicLoading.hide();  
        if(data.type == "OK"){
          $scope.comments.unshift(data.comment);
          $scope.newComment = {};
          //for increment in event comment count
          $scope.event.socials.comments.push(data.comment);
        }
      });
    };

    $scope.removeComment = function(comment){
      var confirmPopup = $ionicPopup.confirm({
         title: 'Delete Comment',
         template: 'You are about to delete the comment ?'
       });
       //delete circle if user click okay
       confirmPopup.then(function(res) {
          if (res) {
              var index = $scope.comments.indexOf(comment);
              $scope.comments.splice(index,1);
              CommentCrud.delete($scope.social_id,comment.id,function(data){
                if (data.type == "OK") {
                  $scope.message = '';
                  $scope.event.socials.comments.splice(0,1);
                }else{
                  $scope.message = data.message;
                }
            });

          }
       });
    };


    $scope.viewProfile = function(user){
      $scope.comingUsersModal.hide();
      $scope.commentsModal.hide();
      if(Contacts.isMe(user.id)){
        $state.go("menu.tab.account");
        return;
      }
      $state.go("menu.tab.account-friend-profile",{accountID : user.id});
    };

    $scope.hide = function(event){
    
     var confirmPopup = $ionicPopup.confirm({
       title: 'Remove Event',
       template: 'Are you sure to remove event permanently?'
     });


      confirmPopup.then(function(res) {
       if (res) {
          //Remove event from localdb
          Events.remove(NB_EVENTS[$scope.activeButton],event ,function(){
            // Remove from local calendar
            CalendarCrud.delete(event);
            
            $ionicLoading.hide(); 
            
            //update related history from localdb
            $scope.ehistory = Events.updateHistoryNode(event,true);
            
            //go back to events tab
            // $state.go('menu.tab.events');
            $ionicHistory.goBack();
            
          });
  
       }
      });
      
    };

    $scope.isActive = function(event){
      if (event) {
        return (event.event_status > 0) && (new Date(event.date + " " + event.time) > new Date());
      }
    };

     //share event
    $scope.share = function(event){
      if ($window.plugins) { 
        var subject = "You've Been Invited!\n\nEvent Name: "+event.title+"\nDate: "+event.date+"\nTime: "+event.time+"\nVenue: "+event.venue+"\n\nCheck out Gurilla App to create and manage your own events, or start your own movement!";
        $window.plugins.socialsharing.share(subject, null, null , 'http://goo.gl/yWSlC6');
      }
    };

    $scope.cancelView = function(){
      $scope.cancelbeforeaccept = false;
      $ionicHistory.goBack();
      // $state.go('menu.tab.events');
    };

    //User cancel the created event which not expired yet
    $scope.cancelEvent = function(event){
      var confirmPopup = $ionicPopup.confirm({
         title: 'Cancel Event',
         template: 'Are you sure , you want to cancel event'
      });


      confirmPopup.then(function(res) {
        if (res) {
          //remove the event from created list

          Events.remove(NB_EVENTS[$scope.activeButton],event,function(){
            //go back to event tabs
            $state.go('menu.tab.events');
          });

          //update the status on server
          EventCrud.cancel(event.id,function(data){
            if (data.type == "OK") {
                CalendarCrud.delete(event);
            }
          });
        }
      }); 
    };


    // // User clicked accept event: Add invited event inside the invited list
    $scope.acceptEvent = function(event){
      $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Updating...'
      });
      //http method to update the event
      EventCrud.accept(true,event.id,function(data){
       
        //Remove event from localdb
        Events.remove(TB_EVENTS.PENDING,event ,function(data){

          //increment the accept count
          event.accept_count++;
          //add this event invited/conifrmed list
          Events.add(TB_EVENTS.INVITED,event);
          
          $ionicLoading.hide(); 
          //go back to events tab
          // $state.go('menu.tab.events');
          $ionicHistory.goBack();
          
          // create event in local calendar
          CalendarCrud.create(event);
        if (data.type == "OK") {
            $rootScope.message = '';

        }else{
          $rootScope.message = data.message;
        }
      });
     });
    };

    //User Click not going button to reject the event invitation
    $scope.rejectEvent = function(event){
      $ionicLoading.show({
        template: '<i class="icon ion-loading-c"></i> Updating...'
      });
      
      //Remove event from localdb
      Events.remove(NB_EVENTS[$scope.activeButton],event ,function(){
        
        $ionicLoading.hide(); 
        //go back to events tab
        // $state.go('menu.tab.events');
        $ionicHistory.goBack();
        
      });
      //http method to update the event
      EventCrud.accept(false,event.id,function(data){
        
        if (data.type == "OK") {
          $rootScope.message = '';
        }else{
          $rootScope.message = data.message;
        }
      });

    };

     //user delete expired event
    $scope.delete = function(event) {
     var confirmPopup = $ionicPopup.confirm({
       title: 'Delete Event',
       template: 'Are you sure you want to delete this event?'
     });

     confirmPopup.then(function(res) {
       if (res) {
        //Remove event from localdb
        Events.remove(NB_EVENTS[$scope.activeButton],event,function(){
          
          $ionicLoading.hide(); 
          
          //remove related history from localdb
          Events.deleteHistoryNode(event);
          
          //go back to events tab
          // $state.go('menu.tab.events');
          $ionicHistory.goBack();
          
        });

        //Delete event from server
        EventCrud.delete(event.id,function(data){
          if (data.type == "OK") {
            console.log('removed');
            // remove event in local calendar
            CalendarCrud.delete(event);
          }else{

          }
         });
       }
     });
    };

    //not used
    $scope.edit = function(event){
      $state.go('menu.tab.event-manage',{'event':event});
    };

    $scope.getEvent = function(){
      EventCrud.detail($stateParams.id,function(data){
          $scope.$broadcast('scroll.refreshComplete');
          if (data.type == "OK") {
            $scope.event = data.event;
            Events.update(NB_EVENTS[$scope.activeButton],$scope.event);
            //update history
            Events.updateHistoryNode($scope.event);
            $ionicLoading.hide();
          }
        });
    };


    if ($scope.para.msgtype) {
      
      $scope.activeButton = ($scope.para.user_status[LocalData.getUserID()])? NNB_EVENTS.INVITED : NNB_EVENTS.PENDING;
      
      if ($scope.para.msgtype == MSGTYPE.CREATED || $scope.para.msgtype == MSGTYPE.EDITED) {
          $ionicLoading.show({
            template: '<i class="icon ion-loading-c"></i> Retrieving event...'
          });

          EventCrud.detail($stateParams.id,function(data){
            $ionicLoading.hide();
            if (data.type == "OK") {
              $scope.event = data.event;
              if ($scope.para.msgtype == MSGTYPE.CREATED ) {
                //add to localdata
                Events.add(TB_EVENTS.PENDING,data.event); 

              }else {
                if($scope.para.user_status[LocalData.getUserID()]){
                  $scope.oldEvent = Events.get("invited",$scope.event.id);
                  if ($scope.oldEvent) {
                      // Update event in local calendar
                      CalendarCrud.update($scope.oldEvent, $scope.event);
                      // Update inside localStorage
                      Events.update(TB_EVENTS.INVITED,$scope.event);
                  }

                }
              }
              //update history
              Events.updateHistoryNode($scope.event);
            }else{
              $scope.message = data.message;
            }
          });
        }else{
         //Handle the cancelled event
         //retrieve the type of event based on whether user had already accept the event or not 
         //accept => event must be in type of invited events otherwise in pending

         //get the event
         $scope.event = Events.get(NB_EVENTS[$scope.activeButton],$stateParams.id);
         if (!$scope.event) {
          $ionicHistory.goBack();
          $scope.message = "Event has been cancelled , you may see detail in events list";
          $scope.cancelbeforeaccept = true;
         }else{
           //update history by telling it is cancelled
           $scope.event.event_status = STATUS.INACTIVE;
           // removed cancel event once user clicked on noti
           CalendarCrud.delete($scope.event);
           Events.updateHistoryNode($scope.event); 
         }
        }
    }else{
       $scope.event = Events.get(NB_EVENTS[$scope.activeButton],$stateParams.id);
    }
  
  $scope.viewPhoto = function(photoId){
    $state.go('menu.tab.event-photo',{'id' : photoId,'event' : $scope.event});
  }; 
  
  //Circle search modal template
  $ionicModal.fromTemplateUrl('templates/modal-view-comings.html', function($ionicModal) {
    $scope.comingUsersModal = $ionicModal;
  }, {
    scope: $scope,
    focusFirstInput: true,
    animation: 'slide-in-up'
  });

   //comments modal template
  $ionicModal.fromTemplateUrl('templates/modal-view-comments.html', function($ionicModal) {
    $scope.commentsModal = $ionicModal;
  }, {
    scope: $scope,
    focusFirstInput: true,
    animation: 'slide-in-up'
  });

  
});

app.controller('EventPhotoCtrl', ['$scope','$state','$stateParams','CommentCrud',
  '$ionicLoading','$anchorScroll','$ionicScrollDelegate','Contacts', function ($scope,$state,$stateParams,CommentCrud,$ionicLoading,$anchorScroll,$ionicScrollDelegate,Contacts) {
  
    
  if (!$stateParams.id) {
    return;
  }

  $scope.event = $stateParams.event;
  $scope.newComment = {
    content : ""
  };
  $scope.photo = {};
  $scope.comments = {};

  $scope.getAllComments = function(page){
    if (page === null) return;
    $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Loading...'
    });
    CommentCrud.read($stateParams.id,page,function(data){
      $ionicLoading.hide();  
      if(data.type == "OK"){
        $scope.photo = data.photo;
        if($scope.comments.length > 0)
          $scope.comments = $scope.comments.concat(data.photo.comments.result);
        else
          $scope.comments = data.photo.comments.result;
        $scope.$broadcast('scroll.refreshComplete');
      }
    });

  };

  $scope.addComment = function(){
   $ionicLoading.show({
      template: '<i class="icon ion-loading-c"></i> Posting...'
   });

   CommentCrud.create($stateParams.id,$scope.newComment,function(data){
      $ionicLoading.hide();  
      if(data.type == "OK"){
        $scope.comments.push(data.comment);
        $scope.newComment = {};
      }
    });
  };

  $scope.loadmore = function(){
    $scope.getAllComments($scope.photo.comments.next);
  };

  $scope.canloadmore = function(){
    if ($scope.photo.comments){
      return ($scope.photo.comments.comment_count > $scope.comments.length);
    }
  };

  $scope.showProfile = function(user) {
    if(Contacts.isMe(user.id)){
        $state.go("menu.tab.account");
        return;
      }
      $state.go("menu.tab.account-friend-profile",{accountID : user.id});
  };

  $scope.gotoAddPost = function() {
      // set the location.hash to the id of
      // the element you wish to scroll to.
     $ionicScrollDelegate.scrollBottom();
    };

  $scope.getAllComments(1);
}]);

