angular.module('gurillaApp.services')

.factory('Events', ['LocalData','TB_EVENTS','EventCrud', function(LocalData,TB_EVENTS,EventCrud)  {
  var working = {};

  var doWriteHistory = function(events){
      var ehistory = LocalData.forceReadObject('ehistory');
      if (Object.keys(ehistory).length === 0) {
        var objects = [];
        objects = objects.concat(events.created);
        objects = objects.concat(events.pending);
        objects = objects.concat(events.invited);

        for (var i = 0; i < objects.length; i++) {
          if(objects[i] && objects[i].id){
            ehistory[objects[i].id] = {
              'active' : objects[i].event_status,
              'version' : objects[i].version,
              'touched' : false,
              'hide' : false
            };  
          }
          
        }
        LocalData.setObject('ehistory',ehistory,true);
      }
      return ehistory;
    };

  return {
    
    set: function(events) {
      LocalData.set('events', JSON.stringify(events));
      return events;
    },
    all: function() {
      var eventsData = LocalData.get('events');
      var eventsList = [];
      if (typeof eventsData !== 'undefined') {
        eventsList = JSON.parse(eventsData);
      }

      return eventsList;
    },
    update : function(type,event){
      var data = LocalData.get('events');

      if (data) {
        events = JSON.parse(LocalData.get('events'));
      }

      angular.forEach(events[type], function(val,index){
        if (val.id == event.id) {
          events[type][index] = event;
          return;
        }
      });

      LocalData.set('events', JSON.stringify(events));
      return events;
    },
    
    add: function(type, event) {
      var events = LocalData.get('events');
      var key = 'events';
      if (!events) {
        key = 'cache_events';
        events = {"created" : [],"invited" : [] , "pending" : []};
      }else{
        events = JSON.parse(events);
      }
      
      if ((typeof events[type] === undefined ) || (events[type].length == 0) ) {
        events[type] = [];
      }

      //make sure it is an array
      //if not make it empty array
      if(!Array.isArray(events[type])){
        events[type]   = [];
      }
      
      events[type].unshift(event);

      LocalData.set(key, JSON.stringify(events));
      console.log('event is added');
      return events;
    },
    get : function(type,id){
      var events = LocalData.get('events');
      var event = undefined;
      //return null if no events data found on localdata
      if (!events) {
        return event;
      }

      events = JSON.parse(events);
      //check whether it is exist or not
      if (!events[type]) {
        return event;
      }
      
      angular.forEach(events[type], function(val,index){
        if (val.id == id) {
          event = val;
        }
      });
      return event;
    },
    remove : function(type,object,callback){
      var events = LocalData.get('events');
      
      if (!events) {
        console.log('Empty events detected');
        LocalData.set('events', JSON.stringify({"created" : [],"invited" : [] , "pending" : []}));
        callback([]);
      }
      
      events = JSON.parse(events);

      if (!object) {
        console.error("Empty object");
        callback(events);  
      }

      for (var i = 0; i < events[type].length; i++) {
        if (events[type][i].id == object.id) {
          events[type].splice(i,1);
          break;
        }
      }

      LocalData.set('events', JSON.stringify(events));
      callback(events[type]);
    },
    writeHistory : function(events){

     return doWriteHistory(events);
      
    },
    updateHistory : function(events){
      var ehistory = LocalData.forceReadObject('ehistory');
      
      var objects = [];
      
      objects = objects.concat(events.created);
      objects = objects.concat(events.pending);
      objects = objects.concat(events.invited);
      for (var i = 0; i < objects.length; i++) {
        if (!ehistory[ objects[i].id ]){
           ehistory[objects[i].id] = {
            'active' : objects[i].event_status,
            'version' : objects[i].version,
            'touched' : false
          };
        }else{
          ehistory[objects[i].id].touched = ehistory[objects[i].id].version == objects[i].version;
        }
      }
      
      LocalData.setObject('ehistory',ehistory);
      console.log('updated event localdata as well');
      
      return ehistory;
    },
    getHistoryNode : function(event){
      var ehistory =  LocalData.getObject('ehistory');
      if (!ehistory && !ehistory[event.id]) {
        return ehistory[event.id];
      }
      return false;
    },
    addHistoryNode : function(event){
      var ehistory =  LocalData.getObject('ehistory');
      if (!ehistory) {
        return false;
      }
      ehistory[event.id] = {
        'active' :  1,
        'version' : 0,
        'touched' : false
      };
      LocalData.setObject('ehistory',ehistory);
    
    },
    deleteHistoryNode : function(event){
      console.log('going to deleted');
      var ehistory =  LocalData.getObject('ehistory');
      ehistory[event.id] = {};
      console.log('deleted from history');
      LocalData.setObject('ehistory',ehistory);
      
    },
    updateHistoryNode : function(event){
      var ehistory =  LocalData.getObject('ehistory');
      if (!ehistory) {
        return false;
      }
      ehistory[event.id] = {
        'active' : event.event_status,
        'version' : event.version | 0,
        'touched' : true,
        'hide' : (arguments[1])? true : false

      };

      LocalData.setObject('ehistory',ehistory);
      console.info('ehistory has been updated');
      return ehistory;
    },
    GetEventsByType : function(type){
      var eventsData = LocalData.get('events');
      var events;

      if (eventsData === undefined) {
        return  events;
      }

      events = JSON.parse(eventsData);
     
      return events[type];
    },
    init: function(){
      EventCrud.read(function(data){
         //create new buffer data
          var e = { created: [], invited: [],pending:[] };
          //fill with coresponding data return by server
          e.created = data.my_events;
          if(data.events){
            e.invited = data.events.confirmed | [];
            e.pending = data.events.pending | [];  
          }else{
            e.invited =  [];
            e.pending =  [];  
          }
          
          //update local data
          LocalData.set('events', JSON.stringify(e));
          //write newly created history
          doWriteHistory(e);
      });
    },
    getCameraOptions : function(source){
      var options = {
          quality: 70,
          sourceType: source,
          encodingType: navigator.camera.EncodingType.JPEG,
          destinationType: navigator.camera.DestinationType.DATA_URL,
          correctOrientation: true,
          targetWidth: 200,
          targetHeight : 200,
          mediaType : navigator.camera.MediaType.PICTURE,
          allowEdit: true,
          saveToPhotoAlbum: false
        };

      return options; 
    }
  };
}])

.factory('EventCrud', ['$http','GURILLA_URL','LocalData','$rootScope','MESSAGE',function ($http,GURILLA_URL,LocalData,$rootScope,MESSAGE) {

  var url = {
    "read" : GURILLA_URL +'events/user',
    "create" : GURILLA_URL +"events",
    "update" : GURILLA_URL +"events",
    "delete" : GURILLA_URL + 'events/delete',
    "detail" : GURILLA_URL  + "events",
    "cancel" : GURILLA_URL +'events/user/cancel',
    "accept" : GURILLA_URL + 'events/confirm',
    "pending" : GURILLA_URL + 'events/user/pending',
    "comings" : GURILLA_URL + 'events'
  };

  var payload = { "token" : LocalData.getToken() };

  var config = { cache: false };

  var getResponse = function(){
      return {"type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  return {
      read : function(callback){
       var response = getResponse();
       
       if(checkOnline()){
          $http.post( url.read , payload ,config).success(function(data, status) {
              if (status == 200) {
                 if (LocalData.get('cache_events')) {
                  LocalData.remove('cache_events');
                 }
                 response = data;
              }
          }).error(function(data, status){
            console.log("ERROR is " + data + "and status is + " + status);
            if(data){
               response.message =  MESSAGE.ERROR.SERVER_UNKNOWN; 
            }
          }).finally(function(){
             callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
      
      },

       readPending : function(callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.post( url.pending , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      detail : function(id,callback){
       
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.post( url.detail + '/' + id , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      search : function(keyword,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
        
          payload.keyword = keyword;

          $http.post( url.search , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }   

      },

      create : function(event,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
           
          payload.event = event;
          //start calling to server
          $http.post( url.create, payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
      },

      update : function(event,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          payload.event = event;
          $http.put( url.update+'/'+event.id , payload ,config).success(function(data, status) {
            if (status == 200) {
              response.type = data.type;
              response.message = data.message;
              response.event = data.event;
            }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
      },

      subscribe : function(unsubscribe,event,callback){

        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
            if (unsubscribe){
              payload.unsubscribe = true;
            }
            payload.event_id = event.id;

            $http.post( url.subscribe , payload ,config).success(function(data, status) {
                if (status == 200) {
                   response = data;
                }
            }).error(function(data, status){
               response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });

        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
      },

      delete : function(id,callback){
       

        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){

          $http.post( url.delete+'/'+id , payload ,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
      },

      cancel : function(id,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
            $http.post( url.cancel+'/'+id , payload ,config).success(function(data, status) {
                if (status == 200) {
                  response.type = data.type;
                  response.message = data.message;
                }
            }).error(function(data){
              response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
      },

      accept : function(flag,id,callback){
       
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){

          payload.coming = flag;
          
          $http.put( url.accept+'/'+id , payload ,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      getComings : function(id,callback){

        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          $http.post( url.comings+'/'+id + '/comings' , payload ,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
                response.users = data.users; 
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }   
      }
  };

}]);