angular.module('gurillaApp.services')

.factory('Interests', function(LocalData,InterestCrud) {

  return {
    init : function(callback){
      InterestCrud.read(function(data){
        LocalData.set('interests', JSON.stringify(data.interests));
        callback(data.interests);
      },function(data){
        console.error(data);
      });
    },
    all: function() {
      var data = LocalData.get('interests');
      var interests = [];
      if (typeof data !== 'undefined') {
        interests = JSON.parse(data);
      }
      return interests;
    },
    get: function(interest) {
      var data = LocalData.get('interests');
      var interests = [];
      if (typeof data !== 'undefined') {
        interests = JSON.parse(data);
      }
      if (typeof interests !== 'undefined') {
        return interests[interest];
      } else {
        return undefined;
      }
    },
    set: function(interests) {
      LocalData.set('interests', JSON.stringify(interests));
      return interests;
    },
    add: function(interest) {
      //get existing localdata
      console.log(interest);
      var data = LocalData.get('interests');
      var interests = [];
      //decode json object unless empty
      if (typeof data !== 'undefined') {
        interests = JSON.parse(data);
      }
      //add newly created interest
      interests.push(interest);
      //encode to json and store to ls
      LocalData.set('interests', JSON.stringify(interests));

      return interests;
    },
    update: function(index,interest) {
      var data = LocalData.get('interests');
      var interests = [];
      if (typeof data !== 'undefined') {
        interests = JSON.parse(data);
      }
      interests[index] = interest;

      LocalData.set('interests', JSON.stringify(interests));

      return interests;
    },
    remove: function(index) {
      var data = LocalData.get('interests');
      var interests = [];
      if (typeof data !== 'undefined') {
        interests = JSON.parse(data);
      }
      
      interests.splice(index,1);

      LocalData.set('interests', JSON.stringify(interests));
      return interests;
    },
    findInterestIndex : function(haystack,needle){
      for (var i = haystack.length - 1; i >= 0; i--) {
        if (haystack[i].id == needle.id){
          return i;
        }
      }
      return -1;
    },
    getOnLength: function(){
       var data = LocalData.get('interests');
       if (!data) {
        return false;
       }
       var interests = JSON.parse(data);
       var count = 0;
       for (var i = interests.length - 1; i >= 0; i--) {
         var options = interests[i].options;
         if (options.notification) {
           count++;
         }
       }
       return count > 0;
    }
  };
})
.factory('InterestCrud', ['$http','GURILLA_URL','LocalData','MESSAGE',function ($http,GURILLA_URL,LocalData,MESSAGE) {
  var url = {
    "read" : GURILLA_URL +'interests/user',
    "create" : GURILLA_URL +"interests",
    "update" : GURILLA_URL +"interests",
    "delete" : GURILLA_URL + 'interests/delete',
    "search" : GURILLA_URL +"interests/search",
    "detail" : GURILLA_URL  + "interests",
    "subscribe" : GURILLA_URL  + "users/subscribe/interest",
  };

  var getPayload = function(){
   return { "token" : LocalData.getToken() };
  };

  var config = { cache: true };

  var getResponse = function(){
      return {"type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  return {
      read : function(callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          $http.post( url.read , getPayload() ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }   
      },

      detail : function(Interest,callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          $http.post( url.detail + '/' + Interest.id , getPayload() ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }        
      },

      search : function(keyword,callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();
          payload.keyword = keyword;

          $http.post( url.search , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
      },

      create : function(interest,callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();
          payload.name = interest.name;
          payload.options = interest.options;

          $http.post( url.create, payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      update : function(interest , callback){

         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();
          payload.name = interest.name;
          payload.options = interest.options;

          

          if(!interest.id){ //update for all
            interest.id = 0;
          }

          console.log(interest);

          $http.put( url.update+'/'+interest.id , payload ,config).success(function(data, status) {
            if (status == 200) {
              response = data;
            }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      subscribe : function(unsubscribe,interest,callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();

          if (unsubscribe){
            payload.unsubscribe = true;
          }
          
          payload.interest_id = interest.id;
          payload.options = interest.options;
         
          $http.post( url.subscribe , payload ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      },

      delete : function(id,callback){
         //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.post( url.delete+'/'+id , getPayload() ,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      }

  };

}]);

