angular.module('gurillaApp.services')
.factory('Moments', ['LocalData', function(LocalData){

  var getLocalMomentsByID = function(id, type){
    var key;
    if (type == "channels") {
        key = "channel_moments";
    }
    var type_moments = LocalData.get(key);

    if (type_moments) {
      type_moments = JSON.parse(type_moments);
      if (type_moments[id]) {
          return type_moments[id];
      }
    }

    return undefined;

  };

  var getMomentsByType = function(type){
    var key;
    if (type == "channels") {
        key = "channel_moments";
    }
    var type_moments = LocalData.get(key);
    
    if (type_moments) {
      return JSON.parse(type_moments);
    }

    return {};
  };

  return {
    getLocalMomentsByChannelID : function(channel_id){  
     return getLocalMomentsByID(channel_id, "channels");
    },

    remove : function(type, channel_id, moment_id){
      var key = type + "_moments";
      var data = getMomentsByType(type+"s");
      var moments = data[channel_id];
      var index = -1;
      for (var i = moments.length - 1; i >= 0; i--) {
        if(moments[i].id == moment_id){
            index = i;
            break;
        } 
      }

      if(index > -1){
        moments.splice(index,1);
        data[channel_id] = moments;
        LocalData.set(key, JSON.stringify(data));
      }

    },

    saveChannelMoments : function(channel_id, data){
      var moments = getMomentsByType("channels");
      
      moments[channel_id] = data;

      LocalData.set('channel_moments', JSON.stringify(moments));

    },

    add: function(moment) {
      var data = LocalData.get('moments');
      var moments = undefined;
      if (typeof data !== 'undefined') {
        moments = JSON.parse(data);
      }
      moments.push(moment);
      
      LocalData.set('moments', JSON.stringify(moments));
      return moments;
    },
    getCameraOptions : function(source){
      var options = {
          quality: 80,
          sourceType: source,
          encodingType: navigator.camera.EncodingType.JPEG,
          destinationType: navigator.camera.DestinationType.DATA_URL,
          correctOrientation: true,
          targetWidth: 400,
          targetHeight : 400,
          mediaType : navigator.camera.MediaType.PICTURE,
          allowEdit: true,
          saveToPhotoAlbum: false
        };

      return options; 
    },
    getVdoOptions : function(source){
      var options = {
          quality: 30,
          destinationType: Camera.DestinationType.FILE_URI,
          sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
          mediaType:Camera.MediaType.VIDEO
        };

      return options; 
    }
  };
}])

.factory('MomentCrud',function (GURILLA_URL,$http,LocalData,MESSAGE,Moments) {
   var url = {
    "moments" : GURILLA_URL + 'moments'
  };

  var config = {headers:  {
      "token" : LocalData.getToken()
    }
  };

  var getResponse = function(){
    return { "type" : "ERROR","message" : "something went wrong"};  
  };

  var getOKResponse = function(){
    return { "type" : "OK"};  
  };

  var checkOnline = function(){
    return navigator.onLine;
  };
  
  return {

    read : function(id,callback){
      //get fresh response object
      var response = getResponse();
      //check network status
      if(checkOnline()){
        //start calling to server

        var response = getResponse();
        $http.get( url.moments + "/" + id, config).success(function(data, status) {
            if (status == 200) {
               response = data;
            }
        }).error(function(data, status){
           response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
        }).finally(function(){
          callback(response);
        });
      }else{
        response.message = MESSAGE.ERROR.OFFLINE;
        callback(response);
      }  
    },

    create : function(moment,callback){
      //get fresh response object
      var response = getResponse();
      //check network status
      if(checkOnline()){
        //start calling to server
        if(moment.image.indexOf("data:image/jpeg;base64,") >= 0 ){
            moment.image = moment.image.replace("data:image/jpeg;base64,", "");
        }

        $http.post( url.moments, moment ,config).success(function(data, status) {
            if (status == 200) {
               response = data;
            }
        }).error(function(data, status){
           response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
        }).finally(function(){
          callback(response);
        });
      }else{
        response.message = MESSAGE.ERROR.OFFLINE;
        callback(response);
      }  
    },
    delete : function(id,callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.delete( url.moments+'/'+id , config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      
    },
    hide : function(id, channel_id, callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.get( url.moments+'/untagchannel/'+id +'/'+ channel_id , config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      
    },
    getUserPhotos : function(id,callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.get(GURILLA_URL+"users/"+id+"/moments",config).success(function(data, status) {
              if (status == 200) {
                response = data;
              }
            }).error(function(data){
              response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
    },

    getMomentsByChannelID : function(id,useCache,callback){
        //get fresh response object        
        var response = getResponse();

        if(useCache){
          var moments = Moments.getLocalMomentsByChannelID(id);
          
          if (moments) {
            response = {"type" : "OK" , "moments" : []};
            response.moments = moments;
            callback(response);
            return;
          }
        }
      


        //check network status
        if(checkOnline()){
          //start calling to server
          $http.get(GURILLA_URL+"moments/photos/channels/"+id,config).success(function(data, status) {
              if (status == 200) {
                response = data;
                Moments.saveChannelMoments(id, data.moments);
              }
            }).error(function(data){
              response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
    }
  };
});

app.factory('CommentCrud', ['$http','GURILLA_URL','LocalData','$rootScope','MESSAGE',function ($http,GURILLA_URL,LocalData,$rootScope,MESSAGE) {
  var url = {
    "socials" : GURILLA_URL +'socials',

  };

  var payload = { "token" : LocalData.getToken() };

  var config = {headers:  {
      "token" : LocalData.getToken()
    }
  };

  var getResponse = function(){
      return {"type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  return {
    read : function(id,page,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          $http.get( url.socials +'/'+id+'/comments/'+page,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
                response.comments = data.comments; 
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });          

        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
    },
    create : function(id,newComment,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){

          $http.post( url.socials +'/'+id+'/comments',newComment,config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
                response.comment = data.comment;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
    },

    delete : function(social_id,comment_id,callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.delete( url.socials+'/'+social_id+"/comments/"+comment_id , config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }  
      
    },
    addLike : function (id,callback) {
      //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){

          $http.post( url.socials +'/'+id+'/likes',{},config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
    },
    removeLike : function (id,callback) {
      //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){

          $http.delete( url.socials +'/'+id+'/likes',config).success(function(data, status) {
              if (status == 200) {
                response.type = data.type;
                response.message = data.message;
              }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        } 
    }
  };
}]);