angular.module('gurillaApp.services')

.factory('PushProcessing', ['$http', 'LocalData', 'GURILLA_URL', function($http, LocalData, GURILLA_URL) {
  function notificationService() {
    if (navigator.onLine) {
      var token = LocalData.getToken();
      var config = { cache: false };
      $http.post(
        GURILLA_URL,{'token': token},config).success(function(data, status) {
        if (data.type == 'OK') {
          if (typeof data.notifications !== 'undefined') {
            //TODO
          }
        } else {
          console.log('Error occured');
        }
      }).error(function(data, status){
        if (status == 0) {
          console.log('Unable to send data. No network connection?');
        } else {
          if (data.code == 403) { /* server said we are forbidden */
            LocalData.clear();
            window.location.href = 'index.html#/login';
          }
        }
      });
    } else {
      console.log('No network connection.');
    }
    setTimeout(notificationService, 30000);
  }

  function onDeviceReady() {
    console.info('NOTIFY  Device is ready.  Registering with GCM server');

    if ((typeof LocalData.get('push_id') !== 'undefined') && (LocalData.get('push_id').length > 0)) {
      console.log('Already registered');
      return;
    } else {
      //register with google GCM server
      var pushNotification = window.plugins.pushNotification;
      var gcmAppID = '476866050946';
      pushNotification.register(gcmSuccessHandler, gcmErrorHandler, {"senderID":gcmAppID,"ecb":"onNotificationGCM"});
    }
  }

  function gcmSuccessHandler(result) {
    console.info('NOTIFY  pushNotification.register succeeded.  Result = '+result);
  }

  function gcmErrorHandler(error) {
    console.error('NOTIFY  '+error);
  }

  return {
    initialize : function () {
      console.info('NOTIFY  initializing');
      document.addEventListener('deviceready', onDeviceReady, false);
      //notificationService();
    },
    registerID : function (id) {
      if (navigator.onLine) {
        var token = LocalData.getToken();
        var config = { cache: false };
        $http.post(
          GURILLA_URL + 'users/addmore',
          {
            'm': 'pushid', 'token': token,
            'push_id': id
          },
          config)
        .success(function(data, status) {
          if (data.type == 'OK') {
            LocalData.set('push_id', id);
            console.log('Updated successfully.');
          } else {
            console.log('Error occured');
          }
        }).error(function(data, status){
          if (status == 0) {
            console.log('Unable to send data. No network connection?');
          } else {
            if (data.code == 403) { /* server said we are forbidden */
              LocalData.clear();
              window.location.href = 'index.html#/login';
            }
          }
        }).finally(function(){
          console.log('Sent update to server.');
        });
      } else {
        console.log('No network connection.');
      }
    }, 
    unregister : function () {
      console.info('unregister')
      var push = window.plugins.pushNotification;
      if (push) {
        push.unregister(function () {
          console.info('unregister success')
        });
      }
    }
  };
}]);


function closeInAppNotification() {
  var notif = document.getElementById('push-notification');
  notif.style.display= 'none';
}

function getBadgeNumber () {
  if ( localStorage['badge'] === undefined ) {
    localStorage['badge'] = 0;
  }
  return localStorage['badge'] = parseInt(localStorage.getItem('badge')) + 1;
}

function showInAppNotification(e) {

  /** Browser **/
  var notif = document.getElementById('push-notification');
  if (notif == null) {
    notif = document.createElement('div');
    notif.id = 'push-notification';
    notif.className = 'balanced-bg light';
    notif.style = 'position:absolute;top:0px;left:0px;height:43px;width:100%;z-index:9999';
    document.body.appendChild(notif);
  }

  notif.onclick = function openEvent() {
    if (typeof e.event_id !== 'undefined') {
      window.location = '#/menu/tab/event/view/invited/'+e.event_id;
    }
    var notif = document.getElementById('push-notification');
    notif.style.display= 'none';
  };
  notif.innerHTML = '';
  notif.innerHTML += '<div style="position:absolute;top:11px;left:11px;overflow:hidden">'+e.message+'</div>';
  notif.innerHTML += '<div style="position:absolute;top:11px;right:11px" onclick="closeInAppNotification()"><i class="icon ion-close"></i></div>';
  notif.style.display= '';

  /** Phone **/
  if (window.plugin) {
    window.plugin.notification.local.add({
      title: e.payload.title,
      message: e.payload.message,
      badge: getBadgeNumber(),
      icon: 'file://img/notification.png',
      autoCancel: true
    });

    console.log("this is on ph");
    window.plugin.notification.local.onclick = function(id, state) {
      window.location.href = '#/menu/tab/events/';
    };
  }
}


window.onNotificationGCM = function(e) {
  console.info(e);
  switch( e.event )
  {
    case 'registered':
      if ( e.regid.length > 0 )
      {
        console.log('REGISTERED with GCM Server -> REGID:' + e.regid + '');
 
        //call back to web service in Angular.  
        //This works for me because in my code I have a factory called
        //    PushProcessing with method registerID
        var elem = angular.element(document.querySelector('[ng-app]'));
        var injector = elem.injector();
        var myService = injector.get('PushProcessing');
        myService.registerID(e.regid);
      }
      break;
 
    case 'message':
      
      // if this flag is set, this notification happened while we were in the foreground.
      // you might want to play a sound to get the user's attention, throw up a dialog, etc.
      if (e.foreground) {
        //we're using the app when a message is received.
        console.log('--INLINE NOTIFICATION--' + '');

        showInAppNotification(e);
      } else {   
        // otherwise we were launched because the user touched a notification in the notification tray.
        if (e.coldstart)
          console.log('--COLDSTART NOTIFICATION--' + '');
        else
          console.log('--BACKGROUND NOTIFICATION--' + '');
          if (window.plugin) {
            console.log("you are on ph");
          }else{
            console.log("on browser");
          }
          
      }
 
      break;
 
    case 'error':
      console.log('ERROR -> MSG:' + e.msg + '');
      break;
 
    default:
      console.log('EVENT -> Unknown, an event was received and we do not know what it is');
      break;
  }
};

