angular.module('gurillaApp.services')

.factory('User', function (GURILLA_URL,$http,LocalData,MESSAGE,$timeout) {

  var url = {
    "addmore" : GURILLA_URL +'users/addmore',
    "update" : GURILLA_URL + 'users',
    "search" : GURILLA_URL + 'users/search',
    "updatelocation" : GURILLA_URL + 'users/updatelocation',
    "photos" : GURILLA_URL + 'photos'
  };


  var getPayload = function(){
    return { "token" : LocalData.getToken() };
  };

  var config = {headers:  {
      "token" : LocalData.getToken()
    }
  };

  var getResponse = function(){
    return { "type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  var getNormalizedPhoneNumber = function(phone){
      //get country code from phonenumber
      var countryCode = countryForE164Number(phone);
      if(!countryCode){
          countryCode =  LocalData.get("country_code"); //default code
      }
      //validate the phonenumber
      if(isValidNumber(phone, countryCode)){
        //valid number will b replaced
        return formatE164(countryCode, phone);
      }
      return phone;
  };

  var findGurillaUser = function(contact){
       var index = -1;
       var gNumbers = LocalData.getObject('normalized_gurilla_phones');

        angular.forEach(contact.phoneNumbers, function(phone){
          phone = getNormalizedPhoneNumber(phone);
          index = gNumbers.indexOf(phone);
        });
        
        return index;
    };

  return {

    isGurillaUser : function(contact){
       var index = findGurillaUser(contact);        
       return index > -1;
    },

    getGurillaUserPhone : function(contact){
        var gNumbers = LocalData.getObject('gurilla_phones');
        var index = findGurillaUser(contact);
        if( index > -1 ){
          return gNumbers[index];
        }

        return undefined;
    },

    updateLocation : function(coords,callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();
          if (coords) {
            payload.lat = coords.lat;
            payload.long = coords.long;
            $http.put( url.updatelocation , payload ,config).success(function(data, status) {
              if (status == 200) {
                response = data;
              }
            }).error(function(data){
              response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });
          }else{
            response.message = "Invalid Location Detected";
            callback(response);
          }
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
    },
    update : function(account,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          var payload = getPayload();

          payload.firstname = account.firstname;
          payload.lastname = account.lastname;
          payload.phone = account.phone;
          payload.dob = account.dob;

          if (account.profile) {
            payload.profile = account.profile;
          }
          payload.profile.gender = account.gender;
          var coords = LocalData.getObject('coords');

          if (coords) {
            payload.profile.lat = coords.lat;
            payload.profile.long = coords.long;
          }else{
            payload.profile.lat = 0;
            payload.profile.long = 0;
          }

          $http.put( url.update , payload ,config).success(function(data, status) {
            if (status == 200) {
              response = data;
            }
          }).error(function(data){
            response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }

    },
    sync : function(callback){
      var response = getResponse();

        if(!checkOnline()){
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
          return;
        }
        LocalData.set('local_contacts', "[]");
        var payload = getPayload();
        //do post to server
        $http.post( url.addmore , payload ,config).success(function(data, status) {
          if (status == 200) {
              //get local data
              var account = JSON.parse(LocalData.get('account'));
              //convert the object to json
              //update the newly contacts
              //we are not using data.contacts anymore
              account.contacts = JSON.stringify(data.contacts);
              // set objects as string in localdb
              // LocalData.set('account',JSON.stringify(account));
              // we are using gurilla_phones now
              LocalData.set("gurilla_phones", JSON.stringify(data["gurilla_phones"]));

              var gurillaPhones = data["gurilla_phones"];
              for (var i = gurillaPhones.length - 1; i >= 0; i--) {
                  gurillaPhones[i] = getNormalizedPhoneNumber(gurillaPhones[i]);
              }
              LocalData.set("normalized_gurilla_phones", JSON.stringify(data["gurilla_phones"]));

              if (!navigator.contactsPhoneNumbers) {
                response.message = "Can not fetch phone contacts";
                callback(response);
                return false;
              }

              
             navigator.contactsPhoneNumbers.list(function(contacts){
                angular.forEach(contacts, function(contact){
                  var phone = [];
                  angular.forEach(contact.phoneNumbers, function(value){
                    phone.push(getNormalizedPhoneNumber(value.normalizedNumber));
                  });
                  contact.phoneNumbers = phone;
                });
                //set objects as string in localdb
                LocalData.set('local_contacts',JSON.stringify(contacts));
              });
          

              response = data;
            }
        }).error(function(data, status){
           response.message = "Unexpected error";
        }).finally(function(){
          callback(response);
        });

        // var response = getResponse();

        // if(!checkOnline()){
        //   response.message = MESSAGE.ERROR.OFFLINE;
        //   callback(response);
        //   return;
        // }
        // LocalData.set('local_contacts', "[]");
        // //do post to server
        // $http.post( url.addmore , getPayload() ,config).success(function(data, status) {
        //   if (status == 200) {
        //        //get local data
        //        var account = JSON.parse(LocalData.get('account'));

        //        //convert the object to json
        //        //update the newly contacts
        //        //we are not using data.contacts anymore
        //        account.contacts = JSON.stringify(data.contacts);

        //        // //set objects as string in localdb
        //        // LocalData.set('account',JSON.stringify(account));
        //       // $timeout(function(){
        //         if (!navigator.contactsPhoneNumbers) {
        //           response.message = "Can not fetch phone contacts";
        //           callback(response);
        //           return false;
        //         }
        //         //we are using gurilla_phones now
        //         var gurillaPhones = data["gurilla_phones"];
        //         //we are using gurilla_phones now
        //         LocalData.set("gurilla_phones", JSON.stringify(data["gurilla_phones"]));

        //         navigator.contactsPhoneNumbers.list(function(contacts){
        //           //remove spaces in ph numbers
        //           for (var i = 0; i < contacts.length; i++) {
        //             var phones = [];
        //             angular.forEach(contacts[i].phoneNumbers, function(value){
        //               var phone = value.normalizedNumber.replace(/\s/g,"");
        //               contacts[i]["gurilla"] = gurillaPhones.indexOf(phone) > 0;
        //               phones.push(phone);
        //             });

        //             contacts[i].phoneNumbers = phones;
        //           }
        //           //set objects as string in localdb
        //           LocalData.set('local_contacts',JSON.stringify(contacts));
        //         });
        //       // }, 100);

        //       response = data;
        //     }
        // }).error(function(data, status){
        //    response.message = "Unexpected error";
        // }).finally(function(){
        //   callback(response);
        // });


    },
    registerGCM : function(gcmID , callback){
      var response = getResponse();
      var payload = getPayload();
      payload.push_id = gcmID;
      //do post to server
      $http.post( url.addmore , payload ,config).success(function(data, status) {
        if (status == 200) {
             response = data;
        }
      }).error(function(data, status){
         response.message = "Unexpected error";
      }).finally(function(){
        callback(response);
      });
    },
    search : function(query,callback){
        //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server

          $http.post( url.search , {"keyword" : query} ,config).success(function(data, status) {
              if (status == 200) {
                 response = data;
              }
          }).error(function(data, status){
             response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
          }).finally(function(){
            callback(response);
          });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
    }

  };
})

.factory('PhotoCrud',function (GURILLA_URL,$http,LocalData,MESSAGE) {
   var url = {
    "read" : GURILLA_URL + 'photos',
    "photos" : GURILLA_URL + 'users'
  };

  var config = {headers:  {
      "token" : LocalData.getToken()
    }
  };

  var getPayload = function(){
    return { "token" : LocalData.getToken() };
  };

  var getResponse = function(){
    return { "type" : "ERROR","message" : "something went wrong"};
  };

  var checkOnline = function(){
    return navigator.onLine;
  };

  return {
    getUserPhotos : function(id,callback){
       //get fresh response object
        var response = getResponse();
        //check network status
        if(checkOnline()){
          //start calling to server
          $http.get(url.photos+"/"+id+"/photos",config).success(function(data, status) {
              if (status == 200) {
                response = data;
              }
            }).error(function(data){
              response.message =  MESSAGE.ERROR.SERVER_UNKNOWN;
            }).finally(function(){
              callback(response);
            });
        }else{
          response.message = MESSAGE.ERROR.OFFLINE;
          callback(response);
        }
    }
  };
});
